#!/bin/python
from datetime import datetime
import copy


class MarketDataBase:
    """
    Abstract class of a market data
    """
    class Side:
        NONE = 0
        BUY = 1
        SELL = 2

    class Depth(object):
        def __init__(self, price=0.0, count=0, volume=0.0):
            """
            Constructor
            """
            self.price = price
            self.count = count
            self.volume = volume

        def copy(self):
            return copy.deepcopy(self)

    @staticmethod
    def parse_side(value):
        """
        Decode the value to Side (BUY/SELL)
        :param value: Integer or string
        :return: Side (NONE, BUY, SELL)
        """
        if type(value) != int:
            value = value.lower()
            if value == 'buy' or value == 'bid' or value == 'b':
                return MarketDataBase.Side.BUY
            elif value == 'sell' or value == 'ask' or value == 's':
                return MarketDataBase.Side.SELL
            else:
                return MarketDataBase.Side.NONE

        if value == 1:
            return MarketDataBase.Side.BUY
        elif value == 2:
            return MarketDataBase.Side.SELL
        else:
            raise Exception("Cannot parse the side (%s)" % value)

    def __init__(self):
        """
        Constructor
        """
        pass


class L2Depth(MarketDataBase):
    """
    L2 price depth. Container of date, time, bid and ask up to 5 levels
    """
    def __init__(self, depth=5):
        """
        Constructor
        :param depth: Number of depth
        """
        MarketDataBase.__init__(self)
        self.date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
        self.depth = depth
        self.bids = [MarketDataBase.Depth() for i in range(0, self.depth)]
        self.asks = [MarketDataBase.Depth() for i in range(0, self.depth)]

    @staticmethod
    def columns():
        """
        Return static columns names
        """
        return ['date_time',
                'b1', 'b2', 'b3', 'b4', 'b5',
                'a1', 'a2', 'a3', 'a4', 'a5',
                'bq1', 'bq2', 'bq3', 'bq4', 'bq5',
                'aq1', 'aq2', 'aq3', 'aq4', 'aq5']

    @staticmethod
    def types():
        """
        Return static column types
        """
        return ['varchar(25)'] + \
               ['decimal(10,5)'] * 10 + \
               ['decimal(20,8)'] * 10

    def values(self):
        """
        Return values in a list
        """
        if self.depth == 5:
            return [self.date_time] + \
                   [b.price for b in self.bids] + \
                   [a.price for a in self.asks] + \
                   [b.volume for b in self.bids] + \
                   [a.volume for a in self.asks]
        else:
            return [self.date_time] + \
                   [b.price for b in self.bids[0:5]] + \
                   [a.price for a in self.asks[0:5]] + \
                   [b.volume for b in self.bids[0:5]] + \
                   [a.volume for a in self.asks[0:5]]

    def sort_bids(self):
        """
        Sorting bids
        :return:
        """
        self.bids.sort(key=lambda x:x.price, reverse=True)
        if len(self.bids) > self.depth:
            self.bids = self.bids[0:self.depth]

    def sort_asks(self):
        """
        Sorting bids
        :return:
        """
        self.asks.sort(key=lambda x:x.price)
        if len(self.asks) > self.depth:
            self.asks = self.asks[0:self.depth]

    def copy(self):
        """
        Copy
        """
        ret = L2Depth(depth=self.depth)
        ret.date_time = self.date_time
        ret.bids = [e.copy() for e in self.bids]
        ret.asks = [e.copy() for e in self.asks]
        return ret

    def is_diff(self, l2_depth):
        """
        Compare the first 5 price depth
        :param l2_depth: Another L2Depth object
        :return: True if they are different
        """
        for i in range(0, 5):
            if abs(self.bids[i].price - l2_depth.bids[i].price) > 1e-09 or \
               abs(self.bids[i].volume - l2_depth.bids[i].volume) > 1e-09:
                return True
            elif abs(self.asks[i].price - l2_depth.asks[i].price) > 1e-09 or \
                abs(self.asks[i].volume - l2_depth.asks[i].volume) > 1e-09:
                return True
        return False

class Trade(MarketDataBase):
    """
    Trade. Container of date, time, trade price, volume and side.
    """
    def __init__(self):
        """
        Constructor
        :param exch: Exchange name
        :param instmt: Instrument name
        :param default_format: Default date time format
        """
        MarketDataBase.__init__(self)
        self.date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
        self.trade_id = ''
        self.trade_price = 0.0
        self.trade_volume = 0.0
        self.trade_side = MarketDataBase.Side.NONE
        self.update_date_time = datetime.utcnow()


    @staticmethod
    def columns():
        """
        Return static columns names
        """
        return ['date_time', 'trade_id', 'trade_price', 'trade_volume', 'trade_side']

    @staticmethod
    def types():
        """
        Return static column types
        """
        return ['varchar(25)', 'text', 'decimal(10,5)', 'decimal(20,8)', 'int']

    def values(self):
        """
        Return values in a list
        """
        return [self.date_time] + \
               [self.trade_id] + [self.trade_price] + [self.trade_volume] + [self.trade_side]


class Snapshot(MarketDataBase):
    """
    Market price snapshot
    """
    class UpdateType:
        NONE = 0
        ORDER_BOOK = 1
        TRADES = 2

    def __init__(self, exchange, instmt_name):
        """
        Constructor
        :param exch: Exchange name
        :param instmt: Instrument name
        :param default_format: Default date time format
        """
        MarketDataBase.__init__(self)

    @staticmethod        
    def columns():
        """
        Return static columns names
        """
        return ['exchange', 'instmt',
                'trade_px', 'trade_volume',
                'b1', 'b2', 'b3', 'b4', 'b5',
                'a1', 'a2', 'a3', 'a4', 'a5',
                'bq1', 'bq2', 'bq3', 'bq4', 'bq5',
                'aq1', 'aq2', 'aq3', 'aq4', 'aq5',
                'order_date_time', 'trades_date_time', 'update_type']
            
    @staticmethod
    def types():
        """
        Return static column types
        """
        return ['varchar(20)', 'varchar(20)', 'decimal(10,5)', 'decimal(20,8)'] + \
               ['decimal(10,5)'] * 10 + \
               ['decimal(20,8)'] * 10 + \
               ['varchar(25)', 'varchar(25)', 'int']
                
    @staticmethod
    def values(exchange_name, instmt_name, l2_depth, last_trade, update_type=UpdateType.NONE):
        """
        Return values in a list
        """        
        return [exchange_name, instmt_name] + \
        [last_trade.trade_price, last_trade.trade_volume] + \
        [b.price for b in l2_depth.bids[0:5]] + \
        [a.price for a in l2_depth.asks[0:5]] + \
        [b.volume for b in l2_depth.bids[0:5]] + \
        [a.volume for a in l2_depth.asks[0:5]] + \
        [l2_depth.date_time, last_trade.date_time, update_type]
        


#!/bin/python
from befh.database_client import DatabaseClient
from befh.market_data import L2Depth, Trade, Snapshot
from datetime import datetime

class ExchangeGateway:
    
    class DataMode:
        """
        Bitwise enum of data mode.
        """
        ALL = 7
        ORDER_BOOK_AND_TRADES_ONLY = 6
        SNAPSHOT_ONLY = 1
        ORDER_BOOK_ONLY = 2
        TRADES_ONLY = 4
        
        @classmethod
        def tostring(cls, val):
            for k,v in vars(cls).iteritems():
                if v==val:
                    return k

        @classmethod
        def fromstring(cls, str):
            return getattr(cls, str.upper(), None)
    
    ############################################################################
    # Static variable data_mode
    # Applied on all gateways whether to record snapshot, order book and trades
    data_mode = DataMode.ALL    
    ############################################################################
    # Static variable 
    # Applied on all gateways whether to record the timestamp in local machine,
    # rather than exchange timestamp given by the API
    is_local_timestamp = True
    ############################################################################
    
    """
    Exchange gateway
    """
    def __init__(self, 
                 api_socket, 
                 db_client=DatabaseClient()):
        """
        Constructor
        :param exchange_name: Exchange name
        :param exchange_api: Exchange API
        :param db_client: Database client
        """
        self.db_client = db_client
        self.api_socket = api_socket

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return ''

    @classmethod
    def get_order_book_table_name(cls, exchange, instmt_name):
        """
        Get order book table name
        :param exchange: Exchange name
        :param instmt_name: Instrument name
        """
        return 'exch_' + exchange.lower() + '_' + instmt_name.lower() + '_book'

    @classmethod
    def get_trades_table_name(cls, exchange, instmt_name):
        """
        Get trades table name
        :param exchange: Exchange name
        :param instmt_name: Instrument name
        """
        return 'exch_' + exchange.lower() + '_' + instmt_name.lower() + '_trades'
        
    @classmethod
    def get_snapshot_table_name(cls):
        return 'exchanges_snapshot'
    
    @classmethod
    def init_snapshot_table(cls, data_mode, db_client):
        if data_mode & ExchangeGateway.DataMode.SNAPSHOT_ONLY:
            db_client.create(cls.get_snapshot_table_name(),
                             Snapshot.columns(),
                             Snapshot.types(),
                             [0,1])
                             
    def init_order_book_table(self, instmt):
        if self.data_mode & ExchangeGateway.DataMode.ORDER_BOOK_ONLY:
            table_name = self.get_order_book_table_name(instmt.get_exchange_name(),
                                                        instmt.get_instmt_name())
            self.db_client.create(table_name,
                                  ['id'] + L2Depth.columns(),
                                  ['int'] + L2Depth.types(),
                                  [0])        
                             
    def init_trades_table(self, instmt):
        if self.data_mode & ExchangeGateway.DataMode.TRADES_ONLY:
            table_name = self.get_trades_table_name(instmt.get_exchange_name(),
                                                    instmt.get_instmt_name())
            self.db_client.create(table_name,
                                  ['id'] + Trade.columns(),
                                  ['int'] + Trade.types(),
                                  [0])  

    def get_order_book_init(self, instmt):
        """
        Initialization method in get_order_book
        :param instmt: Instrument
        :return: Last id
        """
        if self.data_mode & ExchangeGateway.DataMode.ORDER_BOOK_ONLY:
            table_name = self.get_order_book_table_name(instmt.get_exchange_name(),
                                                        instmt.get_instmt_name())            
            self.init_order_book_table(instmt)
            ret = self.db_client.select(table_name,
                                        columns=['id'],
                                        orderby='id desc',
                                        limit=1)
            if len(ret) > 0:
                assert isinstance(ret[0][0], int), "ret[0][0](%s) = %s" % (type(ret[0][0]), ret)
                return ret[0][0]
            else:
                return 0
        else:
            return 0

    def get_trades_init(self, instmt):
        """
        Initialization method in get_trades
        :param instmt: Instrument
        :return: Last id and last exchange trade id
        """
        trade_id = 0
        exch_trade_id = '0'

        if self.data_mode & ExchangeGateway.DataMode.TRADES_ONLY:
            table_name = self.get_trades_table_name(instmt.get_exchange_name(),
                                                    instmt.get_instmt_name())            
            self.init_trades_table(instmt)
            id_ret = self.db_client.select(table=table_name,
                                        columns=['id'],
                                        orderby="id desc",
                                        limit=1)
            trade_id_ret = self.db_client.select(table=table_name,
                                           columns=['id', 'trade_id'],
                                           orderby="id desc",
                                           limit=1)
    
            if len(id_ret) > 0 and len(trade_id_ret) > 0:
                trade_id = id_ret[0][0]
                exch_trade_id = trade_id_ret[0][1]

                # Convert back to proper type
                if isinstance(trade_id, str):
                    trade_id = int(trade_id)

                if isinstance(exch_trade_id, int):
                    exch_trade_id = str(exch_trade_id)

        return trade_id, exch_trade_id
    
    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        return []

    def insert_order_book(self, instmt):
        """
        Insert order book row into the database client
        :param instmt: Instrument
        """
        # If local timestamp indicator is on, assign the local timestamp again
        if self.is_local_timestamp:
            instmt.get_l2_depth().date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
        
        # Update the snapshot
        if self.data_mode & ExchangeGateway.DataMode.SNAPSHOT_ONLY and \
           instmt.get_l2_depth() is not None:
            self.db_client.insert(table=self.get_snapshot_table_name(),
                                  columns=Snapshot.columns(),
                                  types=Snapshot.types(),
                                  values=Snapshot.values(instmt.get_exchange_name(),
                                                         instmt.get_instmt_name(),
                                                         instmt.get_l2_depth(),
                                                         Trade() if instmt.get_last_trade() is None else instmt.get_last_trade(),
                                                         Snapshot.UpdateType.ORDER_BOOK),
                                  primary_key_index=[0,1],
                                  is_orreplace=True,
                                  is_commit=not(self.data_mode & ExchangeGateway.DataMode.ORDER_BOOK_ONLY))
            
        # Update its order book table
        if self.data_mode & ExchangeGateway.DataMode.ORDER_BOOK_ONLY:
            self.db_client.insert(table=instmt.get_order_book_table_name(),
                                  columns=['id'] + L2Depth.columns(),
                                  types=['int'] + L2Depth.types(),
                                  values=[instmt.get_order_book_id()] + instmt.get_l2_depth().values())

    def insert_trade(self, instmt, trade):
        """
        Insert trade row into the database client
        :param instmt: Instrument
        """
        # If the instrument is not recovered, skip inserting into the table
        if not instmt.get_recovered():
            return
        
        # If local timestamp indicator is on, assign the local timestamp again
        if self.is_local_timestamp:
            trade.date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
        
        # Set the last trade to the current one
        instmt.set_last_trade(trade)

        # Update the snapshot
        if self.data_mode & ExchangeGateway.DataMode.SNAPSHOT_ONLY and \
           instmt.get_l2_depth() is not None and \
           instmt.get_last_trade() is not None:
            self.db_client.insert(table=self.get_snapshot_table_name(),
                                  columns=Snapshot.columns(),
                                  values=Snapshot.values(instmt.get_exchange_name(),
                                                         instmt.get_instmt_name(),
                                                         instmt.get_l2_depth(),
                                                         instmt.get_last_trade(),
                                                         Snapshot.UpdateType.TRADES),
                                  types=Snapshot.types(),
                                  primary_key_index=[0,1],
                                  is_orreplace=True,
                                  is_commit=not(self.data_mode & ExchangeGateway.DataMode.TRADES_ONLY))
        
        # Update its trade table
        if self.data_mode & ExchangeGateway.DataMode.TRADES_ONLY:
            self.db_client.insert(table=instmt.get_trades_table_name(),
                                  columns=['id']+Trade.columns(),
                                  types=['int']+Trade.types(),
                                  values=[instmt.get_trade_id()]+trade.values())



from befh.database_client import DatabaseClient
from befh.util import Logger
import threading
import os
import csv

class FileClient(DatabaseClient):
    """
    File client
    """

    class Operator:
        UNKNOWN = 0
        EQUAL = 1
        NOT_EQUAL = 2
        GREATER = 3
        GREATER_OR_EQUAL = 4
        SMALLER = 5
        SMALLER_OR_EQUAL = 6

    def __init__(self, dir=os.getcwd()):
        """
        Constructor
        """
        DatabaseClient.__init__(self)
        self.lock = threading.Lock()
        self.file_mapping = dict()

        if dir is None or dir == '':
            raise Exception("FileClient does not accept empty directory.")
        if dir[-1] != '\\':
            dir += '\\'

        self.file_directory = dir

    @staticmethod
    def convert_to(from_str, to_type):
        """
        Convert the element to the given type
        """
        if to_type is int:
            return int(from_str)
        elif to_type is float:
            return float(from_str)
        else:
            return from_str


    def create(self, table, columns, types, is_ifnotexists=True):
        """
        Create table in the database
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param is_ifnotexists: Create table if not exists keyword
        """
        file_path = self.file_directory + table + ".csv"
        columns = [e.split(' ')[0] for e in columns]
        if len(columns) != len(types):
            return False

        self.lock.acquire()
        if os.path.isfile(file_path):
            Logger.info(self.__class__.__name__, "File (%s) has been created already." % file_path)
        else:
            with open(file_path, 'w+') as csvfile:
                csvfile.write(','.join(["\"" + e + "\"" for e in columns])+'\n')

        self.lock.release()
        return True

    def insert(self, table, columns, types, values, primary_key_index=[], is_orreplace=False, is_commmit=True):
        """
        Insert into the table
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param values: Value array
        :param primary_key_index: An array of indices of primary keys in columns,
                          e.g. [0] means the first column is the primary key
        :param is_orreplace: Indicate if the query is "INSERT OR REPLACE"
        """
        ret = True
        file_path = self.file_directory + table + ".csv"
        if len(columns) != len(values):
            return False

        self.lock.acquire()
        if not os.path.isfile(file_path):
            ret = False
        else:
            with open(file_path, "a+") as csvfile:
                writer = csv.writer(csvfile, lineterminator='\n', quotechar='\"', quoting=csv.QUOTE_NONNUMERIC)
                writer.writerow(values)
        self.lock.release()

        if not ret:
            raise Exception("File (%s) has not been created.")

        return True

    def select(self, table, columns=['*'], condition='', orderby='', limit=0, isFetchAll=True):
        """
        Select rows from the table.
        Currently the method only processes the one column ordering and condition
        :param table: Table name
        :param columns: Selected columns
        :param condition: Where condition
        :param orderby: Order by condition
        :param limit: Rows limit
        :param isFetchAll: Indicator of fetching all
        :return Result rows
        """
        file_path = self.file_directory + table + ".csv"
        is_all_columns = len(columns) == 1 and columns[0] == '*'
        csv_field_names = []
        columns = [e.split(' ')[0] for e in columns]
        ret = []
        is_error = False

        # Preparing condition
        if condition != '':
            condition = condition.replace('=', '==')
            condition = condition.replace('!==', '!=')
            condition = condition.replace('>==', '>=')
            condition = condition.replace('<==', '<=')

        self.lock.acquire()
        if not os.path.isfile(file_path):
            is_error = True
        else:
            with open(file_path, "r") as csvfile:
                reader = csv.reader(csvfile, lineterminator='\n', quotechar='\"', quoting=csv.QUOTE_NONNUMERIC)
                csv_field_names = next(reader, None)
                for col in columns:
                    if not is_all_columns and col not in csv_field_names:
                        raise Exception("Field (%s) is not in the table." % col)

                for csv_row in reader:
                    # Filter by condition statement
                    is_selected = True
                    if condition != '':
                        condition_eval = condition
                        for i in range(0, len(csv_field_names)):
                            key = csv_field_names[i]
                            value = csv_row[i]
                            if condition_eval.find(key) > -1:
                                condition_eval = condition_eval.replace(key, str(value))
                        is_selected = eval(condition_eval)

                    if is_selected:
                        ret.append(list(csv_row))

        self.lock.release()

        if is_error:
            raise Exception("File (%s) has not been created.")

        if orderby != '':
            # Sort the result
            field = orderby.split(' ')[0].strip()
            asc_val = orderby.split(' ')[1].strip() if len(orderby.split(' ')) > 1 else 'asc'
            if asc_val != 'asc' and asc_val != 'desc':
                raise Exception("Incorrect orderby in select statement (%s)." % orderby)
            elif field not in csv_field_names:
                raise Exception("Field (%s) is not in the table." % col)

            field_index = csv_field_names.index(field)
            ret = sorted(ret, key=lambda x:x[field_index], reverse=(asc_val == 'desc'))

        if limit > 0:
            # Trim the result by the limit
            ret = ret[:limit]

        if not is_all_columns:
            field_index = [csv_field_names.index(x) for x in columns]
            ret = [[row[i] for i in field_index] for row in ret]

        return ret

    def delete(self, table, condition='1==1'):
        """
        Delete rows from the table
        :param table: Table name
        :param condition: Where condition
        """
        raise Exception("Deletion is not supported in file client.")




from befh.ws_api_socket2 import WebSocketApiClient
from befh.market_data import L2Depth, Trade
from befh.exchange import ExchangeGateway
from befh.instrument import Instrument
from befh.sql_client_template import SqlClientTemplate
from befh.util import Logger
import time
import threading
import json
from functools import partial
from datetime import datetime


class ExchGwApiHuobi(WebSocketApiClient):
    """
    Exchange socket
    """
    def __init__(self):
        """
        Constructor
        """
        WebSocketApiClient.__init__(self, 'Huobi')
        
    @classmethod
    def get_order_book_timestamp_field_name(cls):
        return ''
        
    @classmethod
    def get_trades_timestamp_field_name(cls):
        return 'time'
    
    @classmethod
    def get_bids_field_name(cls):
        return 'topBids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'topAsks'
        
    @classmethod
    def get_trade_side_field_name(cls):
        return 'direction'
        
    @classmethod
    def get_trade_id_field_name(cls):
        return 'tradeId'
        
    @classmethod
    def get_trade_price_field_name(cls):
        return 'price'        
        
    @classmethod
    def get_trade_volume_field_name(cls):
        return 'amount'
        
    @classmethod
    def get_link(cls):
        return 'hq.huobi.com'

    @classmethod
    def get_trades_subscription_string(cls, instmt):
        return {"symbolList":
                    {"tradeDetail":[{"symbolId":instmt.get_instmt_code()}]},
                "version":1,
                "msgType":"reqMsgSubscribe",
                "requestIndex":157934767
                }

    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        l2_depth = instmt.get_l2_depth()
        keys = list(raw.keys())
        if cls.get_bids_field_name() in keys and \
           cls.get_asks_field_name() in keys:
            
            # Date time
            l2_depth.date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
            
            # Bids
            bids = raw[cls.get_bids_field_name()][0]
            prices = bids['price']
            amounts = bids['amount']
            for i in range(0, len(prices)):
                l2_depth.bids[i].price = prices[i]
                l2_depth.bids[i].volume = amounts[i]
                
            # Asks
            asks = raw[cls.get_asks_field_name()][0]
            prices = asks['price']
            amounts = asks['amount']
            for i in range(0, len(prices)):
                l2_depth.asks[i].price = prices[i]
                l2_depth.asks[i].volume = amounts[i]

        else:
            raise Exception('Does not contain order book keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))
        
        return l2_depth        

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        keys = list(raw.keys())
        ret = []

        if cls.get_trades_timestamp_field_name() in keys and \
           cls.get_trade_id_field_name() in keys and \
           cls.get_trade_side_field_name() in keys and \
           cls.get_trade_price_field_name() in keys and \
           cls.get_trade_volume_field_name() in keys:

            tradeTime = raw[cls.get_trades_timestamp_field_name()]
            tradeId = raw[cls.get_trade_id_field_name()]
            tradeSide = raw[cls.get_trade_side_field_name()]
            tradePrice = raw[cls.get_trade_price_field_name()]
            tradeQty = raw[cls.get_trade_volume_field_name()]

            for i in range(0, len(tradeTime)):
                trade = Trade()

                # Date time
                # timestamp = tradeTime[i]
                # timestamp = datetime.utcfromtimestamp(timestamp).strftime("%Y%m%d %H:%M:%S.%f")
                # trade.date_time = timestamp

                # Trade side
                if tradeSide[i] == 3:
                    trade.trade_side = Trade.parse_side(1)
                elif tradeSide[i] == 4:
                    trade.trade_side = Trade.parse_side(2)
                else:
                    trade.trade_side = Trade.parse_side(tradeSide[i])

                # Trade id
                trade.trade_id = str(tradeId[i])

                # Trade price
                trade.trade_price = tradePrice[i]

                # Trade volume
                trade.trade_volume = tradeQty[i]

                ret.append(trade)

        else:
            raise Exception('Does not contain trade keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))

        return ret


class ExchGwHuobi(ExchangeGateway):
    """
    Exchange gateway
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwApiHuobi(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'Huobi'

    def on_open_handler(self, instmt, ws):
        """
        Socket on open handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is subscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        if not instmt.get_subscribed():
            ws.send(self.api_socket.get_trades_subscription_string(instmt))
            instmt.set_subscribed(True)

    def on_close_handler(self, instmt, ws):
        """
        Socket on close handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is unsubscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        instmt.set_subscribed(False)

    def on_message_handler(self, instmt, message):
        """
        Incoming message handler
        :param instmt: Instrument
        :param message: Message
        """
        payload = message['payload']
        if len(payload) > 0 and 'symbolId' in message.keys() and \
            message['symbolId'] == instmt.get_instmt_code():
            instmt.set_prev_l2_depth(instmt.get_l2_depth().copy())
            self.api_socket.parse_l2_depth(instmt, payload)
            if instmt.get_l2_depth().is_diff(instmt.get_prev_l2_depth()):
                instmt.incr_order_book_id()
                self.insert_order_book(instmt)

            trades = self.api_socket.parse_trade(instmt, payload)
            for trade in trades:
                if trade.trade_id != instmt.get_exch_trade_id():
                    instmt.incr_trade_id()
                    instmt.set_exch_trade_id(trade.trade_id)
                    self.insert_trade(instmt, trade)

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_l2_depth(L2Depth(5))
        instmt.set_prev_l2_depth(L2Depth(5))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                       instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                               instmt.get_instmt_name()))
        instmt.set_order_book_id(self.get_order_book_init(instmt))
        trade_id, last_exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(last_exch_trade_id)
        return [self.api_socket.connect(self.api_socket.get_link(),
                                        on_message_handler=partial(self.on_message_handler, instmt),
                                        on_open_handler=partial(self.on_open_handler, instmt),
                                        on_close_handler=partial(self.on_close_handler, instmt))]
                                        

if __name__ == '__main__':
    exchange_name = 'Huobi'
    instmt_name = 'BTCCNY'
    instmt_code = 'btccny'
    instmt = Instrument(exchange_name, instmt_name, instmt_code)
    db_client = SqlClientTemplate()
    Logger.init_log()
    exch = ExchGwHuobi(db_client)
    td = exch.start(instmt)




from befh.ws_api_socket import WebSocketApiClient
from befh.market_data import L2Depth, Trade
from befh.exchange import ExchangeGateway
from befh.instrument import Instrument
from befh.sql_client_template import SqlClientTemplate
from befh.util import Logger
import time
import threading
import json
from functools import partial
from datetime import datetime


class ExchGwApiBitstamp(WebSocketApiClient):
    """
    Exchange socket
    """
    def __init__(self):
        """
        Constructor
        """
        WebSocketApiClient.__init__(self, 'Bitstamp')
        
    @classmethod
    def get_trades_timestamp_field_name(cls):
        return 'timestamp'
    
    @classmethod
    def get_bids_field_name(cls):
        return 'bids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'asks'
        
    @classmethod
    def get_trade_side_field_name(cls):
        return 'type'
        
    @classmethod
    def get_trade_id_field_name(cls):
        return 'id'
        
    @classmethod
    def get_trade_price_field_name(cls):
        return 'price'        
        
    @classmethod
    def get_trade_volume_field_name(cls):
        return 'amount'   
        
    @classmethod
    def get_link(cls):
        return 'ws://ws.pusherapp.com/app/de504dc5763aeef9ff52?protocol=7'

    @classmethod
    def get_order_book_subscription_string(cls, instmt):
        if instmt.get_instmt_code() == "":
            return json.dumps({"event":"pusher:subscribe","data":{"channel":"order_book"}})
        else:
            return json.dumps({"event":"pusher:subscribe","data":{"channel":"order_book_%s" % instmt.get_instmt_code()}})

    @classmethod
    def get_trades_subscription_string(cls, instmt):
        if instmt.get_instmt_code() == "":
            return json.dumps({"event":"pusher:subscribe","data":{"channel":"live_trades"}})        
        else:
            return json.dumps({"event":"pusher:subscribe","data":{"channel":"live_trades_%s" % instmt.get_instmt_code()}})        
            
    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        l2_depth = instmt.get_l2_depth()
        keys = list(raw.keys())
        if cls.get_bids_field_name() in keys and \
           cls.get_asks_field_name() in keys:
            
            # Date time
            l2_depth.date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
            
            # Bids
            bids = raw[cls.get_bids_field_name()]
            for i in range(0, len(bids)):
                l2_depth.bids[i].price = float(bids[i][0]) if type(bids[i][0]) != float else bids[i][0]
                l2_depth.bids[i].volume = float(bids[i][1]) if type(bids[i][1]) != float else bids[i][1]   
            bids = sorted(bids, key=lambda x: x[0], reverse=True)
                
            # Asks
            asks = raw[cls.get_asks_field_name()]
            for i in range(0, len(asks)):
                l2_depth.asks[i].price = float(asks[i][0]) if type(asks[i][0]) != float else asks[i][0]
                l2_depth.asks[i].volume = float(asks[i][1]) if type(asks[i][1]) != float else asks[i][1]            
            asks = sorted(asks, key=lambda x: x[0])
        else:
            raise Exception('Does not contain order book keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))
        
        return l2_depth        

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        trade = Trade()
        keys = list(raw.keys())
        
        if cls.get_trades_timestamp_field_name() in keys and \
           cls.get_trade_id_field_name() in keys and \
           cls.get_trade_side_field_name() in keys and \
           cls.get_trade_price_field_name() in keys and \
           cls.get_trade_volume_field_name() in keys:
        
            # Date time
            date_time = float(raw[cls.get_trades_timestamp_field_name()])
            trade.date_time = datetime.utcfromtimestamp(date_time).strftime("%Y%m%d %H:%M:%S.%f")            
            
            # Trade side
            # Buy = 0
            # Side = 1
            trade.trade_side = Trade.parse_side(raw[cls.get_trade_side_field_name()] + 1)
                
            # Trade id
            trade.trade_id = raw[cls.get_trade_id_field_name()]
            
            # Trade price
            trade.trade_price = raw[cls.get_trade_price_field_name()]            
            
            # Trade volume
            trade.trade_volume = raw[cls.get_trade_volume_field_name()]                        
        else:
            raise Exception('Does not contain trade keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))        

        return trade        


class ExchGwBitstamp(ExchangeGateway):
    """
    Exchange gateway
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwApiBitstamp(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'Bitstamp'

    def on_open_handler(self, instmt, ws):
        """
        Socket on open handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is subscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        if not instmt.get_subscribed():
            ws.send(self.api_socket.get_order_book_subscription_string(instmt))
            ws.send(self.api_socket.get_trades_subscription_string(instmt))            
            instmt.set_subscribed(True)

    def on_close_handler(self, instmt, ws):
        """
        Socket on close handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is unsubscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        instmt.set_subscribed(False)

    def on_message_handler(self, instmt, message):
        """
        Incoming message handler
        :param instmt: Instrument
        :param message: Message
        """
        keys = message.keys()
        if 'event' in keys and message['event'] in ['data', 'trade'] and 'channel' in keys and 'data' in keys:
            channel_name = message['channel']
            if (instmt.get_instmt_code() == "" and channel_name == "order_book") or \
               (instmt.get_instmt_code() != "" and channel_name == "order_book_%s" % instmt.get_instmt_code()):
                instmt.set_prev_l2_depth(instmt.get_l2_depth().copy())
                self.api_socket.parse_l2_depth(instmt, json.loads(message['data']))
                if instmt.get_l2_depth().is_diff(instmt.get_prev_l2_depth()):
                    instmt.incr_order_book_id()
                    self.insert_order_book(instmt)    
            elif (instmt.get_instmt_code() == "" and channel_name == "live_trades") or \
                 (instmt.get_instmt_code() != "" and channel_name == "live_trades_%s" % instmt.get_instmt_code()):
                    trade = self.api_socket.parse_trade(instmt, json.loads(message['data']))
                    if trade.trade_id != instmt.get_exch_trade_id():
                        instmt.incr_trade_id()
                        instmt.set_exch_trade_id(trade.trade_id)
                        self.insert_trade(instmt, trade)     

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_l2_depth(L2Depth(20))
        instmt.set_prev_l2_depth(L2Depth(20))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                       instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                               instmt.get_instmt_name()))
        instmt.set_order_book_id(self.get_order_book_init(instmt))
        trade_id, last_exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(last_exch_trade_id)
        return [self.api_socket.connect(self.api_socket.get_link(),
                                        on_message_handler=partial(self.on_message_handler, instmt),
                                        on_open_handler=partial(self.on_open_handler, instmt),
                                        on_close_handler=partial(self.on_close_handler, instmt))]
                                        

if __name__ == '__main__':
    exchange_name = 'Bitstamp'
    instmt_name = 'BTCUSD'
    instmt_code = ''
    instmt = Instrument(exchange_name, instmt_name, instmt_code)
    db_client = SqlClientTemplate()
    Logger.init_log()
    exch = ExchGwBitstamp(db_client)
    td = exch.start(instmt)




from befh.api_socket import ApiSocket
from befh.util import Logger
import threading
from functools import partial
from socketIO_client import SocketIO

class WebSocketApiClient(ApiSocket):
    """
    Generic REST API call
    """
    def __init__(self, id):
        """
        Constructor
        :param id: Socket id
        """
        ApiSocket.__init__(self)
        self.ws = None              # Web socket
        self.id = id
        self.wst = None             # Web socket thread
        self._connected = False
        self.on_message_handlers = []
        self.on_open_handlers = []
        self.on_close_handlers = []
        self.on_error_handlers = []

    def connect(self, url,
                on_message_handler=None,
                on_open_handler=None,
                on_close_handler=None,
                on_error_handler=None):
        """
        :param url: Url link
        :param on_message_handler: Message handler which take the message as
                           the first argument
        :param on_open_handler: Socket open handler which take the socket as
                           the first argument
        :param on_close_handler: Socket close handler which take the socket as
                           the first argument
        :param on_error_handler: Socket error handler which take the socket as
                           the first argument and the error as the second
                           argument
        """
        Logger.info(self.__class__.__name__, "Connecting to socket <%s>..." % self.id)
        if on_message_handler is not None:
            self.on_message_handlers.append(on_message_handler)
        if on_open_handler is not None:
            self.on_open_handlers.append(on_open_handler)
        if on_close_handler is not None:
            self.on_close_handlers.append(on_close_handler)
        if on_error_handler is not None:
            self.on_error_handlers.append(on_error_handler)

        if not self._connected:
            self.wst = threading.Thread(target=lambda: self.run(url, 80))
            self.wst.start()

        return self.wst

    def run(self, url, port):
        self.ws = SocketIO(url, port)
        self.ws.on('connect', self.__on_open)
        self.ws.on('request', self.__on_message)
        self.ws.on('message', self.__on_message)
        self.ws.on('disconnect', self.__on_close)
        while True:
            self.ws.wait(1)

    def send(self, msg):
        """
        Send message
        :param msg: Message
        :return:
        """
        self.ws.emit('request', msg)

    def __on_message(self, m):
        for handler in self.on_message_handlers:
            handler(m)

    def __on_open(self):
        Logger.info(self.__class__.__name__, "Socket <%s> is opened." % self.id)
        self._connected = True
        for handler in self.on_open_handlers:
            handler(self)
        
    def __on_close(self):
        Logger.info(self.__class__.__name__, "Socket <%s> is closed." % self.id)
        self._connected = False
        for handler in self.on_close_handlers:
            handler(self)
        
    def __on_error(self, error):
        Logger.info(self.__class__.__name__, "Socket <%s> error:\n %s" % (self.id, error))
        for handler in self.on_error_handlers:
            handler(self, error)



from befh.database_client import DatabaseClient
from befh.util import Logger
import threading

class SqlClient(DatabaseClient):
    """
    Sql client
    """
    @classmethod
    def replace_keyword(cls):
        return 'replace into'

    def __init__(self):
        """
        Constructor
        """
        DatabaseClient.__init__(self)
        self.conn = None
        self.cursor = None
        self.lock = threading.Lock()

    def execute(self, sql):
        """
        Execute the sql command
        :param sql: SQL command
        """
        return True

    def commit(self):
        """
        Commit
        """
        return True

    def fetchone(self):
        """
        Fetch one record
        :return Record
        """
        return []

    def fetchall(self):
        """
        Fetch all records
        :return Record
        """
        return []

    def create(self, table, columns, types, primary_key_index=[], is_ifnotexists=True):
        """
        Create table in the database
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param is_ifnotexists: Create table if not exists keyword
        """
        if len(columns) != len(types):
            raise Exception("Incorrect create statement. Number of columns and that of types are different.\n%s\n%s" % \
                (columns, types))

        column_names = ''
        for i in range(0, len(columns)):
            column_names += '%s %s,' % (columns[i], types[i])
        
        if len(primary_key_index) > 0:
            column_names += 'PRIMARY KEY (%s)' % (",".join([columns[e] for e in primary_key_index]))
        else:
            column_names = column_names[0:len(column_names)-1]

        if is_ifnotexists:
            sql = "create table if not exists %s (%s)" % (table, column_names)
        else:
            sql = "create table %s (%s)" % (table, column_names)

        self.lock.acquire()
        
        try:
            self.execute(sql)
        except Exception as e:
            raise Exception("Error in create statement (%s).\nError: %s\n" % (sql, e))
            
        self.commit()
        self.lock.release()
        return True

    def insert(self, table, columns, types, values, primary_key_index=[], is_orreplace=False, is_commit=True):
        """
        Insert into the table
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param values: Value array
        :param primary_key_index: An array of indices of primary keys in columns,
                          e.g. [0] means the first column is the primary key
        :param is_orreplace: Indicate if the query is "INSERT OR REPLACE"
        """
        if len(columns) != len(values):
            return False

        column_names = ','.join(columns)
        value_string = ','.join([SqlClient.convert_str(e) for e in values])
        if is_orreplace:
            sql = "%s %s (%s) values (%s)" % (self.replace_keyword(), table, column_names, value_string)
        else:
            sql = "insert into %s (%s) values (%s)" % (table, column_names, value_string)

        self.lock.acquire()
        try:
            self.execute(sql)
            if is_commit:
                self.commit()
        except Exception as e:
            Logger.info(self.__class__.__name__, "SQL error: %s\nSQL: %s" % (e, sql))
        self.lock.release()
        return True

    def select(self, table, columns=['*'], condition='', orderby='', limit=0, isFetchAll=True):
        """
        Select rows from the table
        :param table: Table name
        :param columns: Selected columns
        :param condition: Where condition
        :param orderby: Order by condition
        :param limit: Rows limit
        :param isFetchAll: Indicator of fetching all
        :return Result rows
        """
        sql = "select %s from %s" % (','.join(columns), table)
        if len(condition) > 0:
            sql += " where %s" % condition

        if len(orderby) > 0:
            sql += " order by %s" % orderby

        if limit > 0:
            sql += " limit %d" % limit

        self.lock.acquire()
        self.execute(sql)
        if isFetchAll:
            ret = self.fetchall()
            self.lock.release()
            return ret
        else:
            ret = self.fetchone()
            self.lock.release()
            return ret

    def delete(self, table, condition='1==1'):
        """
        Delete rows from the table
        :param table: Table name
        :param condition: Where condition
        """
        sql = "delete from %s" % table
        if len(condition) > 0:
            sql += " where %s" % condition

        self.lock.acquire()
        self.execute(sql)
        self.commit()
        self.lock.release()
        return True



#!/bin/python

from befh.sql_client import SqlClient
from befh.util import Logger

class SqlClientTemplate(SqlClient):
    """
    Sql client template
    """
    def __init__(self):
        """
        Constructor
        """
        SqlClient.__init__(self)

    def connect(self, **kwargs):
        """
        Connect
        """
        return True
        
    def execute(self, sql):
        """
        Execute the sql command
        :param sql: SQL command
        """
        Logger.info(self.__class__.__name__, "Execute command = %s" % sql)
        
    def commit(self):
        """
        Commit
        """    
        pass
        
    def fetchone(self):
        """
        Fetch one record
        :return Record
        """        
        return []

    def fetchall(self):
        """
        Fetch all records
        :return Record
        """        
        return []




from befh.database_client import DatabaseClient
from befh.util import Logger
import threading
import re
import numpy
from qpython import qconnection
from qpython.qcollection import QTable, QKeyedTable, QList

class KdbPlusClient(DatabaseClient):
    """
    Kdb+ Client
    """
    @classmethod
    def parse_condition(cls, condition):
        """
        Parsing the condition statement to kdb format
        :param condition: Condition statement string
        :return: Parsed condition statement
        """
        condition_statement = ''
        condition_items = re.split(' and | or ', condition)
        condition_logic = re.compile('and|or').findall(condition)
        for i in range(0, len(condition_items)):
            if condition_items[i].find("\"") > -1:
                condition_items[i] = re.sub('(.*)([>=|<=|!=|=|>|<])(\s*)"(.*)"', '\\1\\2\\3`\\4', condition_items[i])
            else:
                condition_items[i] = re.sub('(.*)([>=|<=|!=|=|>|<])(\s*)(.*)', '\\1\\2\\3\\4', condition_items[i])

            condition_statement += condition_items[i]

            if i < len(condition_logic):
                condition_statement += ',' if condition_logic[i] == 'and' else '|'

        return condition_statement

    @classmethod
    def convert_type(cls, type):
        """
        Normalize the sql type
        :param type: SQL type
        :return: Type
        """
        if type.find('text') > -1 or type.find('varchar') > -1:
            return str
        elif type == 'float' or type == 'double' or type.find('decimal') > -1:
            return float
        elif type == 'int':
            return int
        else:
            raise Exception("Failed to convert type (%s)" % type)

    @classmethod
    def decode_qtypes(cls, value):
        """
        Decode all the QCollection items to normal python types
        """
        if isinstance(value, numpy.bytes_):
            return value.decode("utf-8")
        elif isinstance(value, list):
            return value
        else:
            return value.item()

    def __init__(self):
        """
        Constructor
        """
        DatabaseClient.__init__(self)
        self.conn = None
        self.lock = threading.Lock()

    def connect(self, **kwargs):
        """
        Connect
        :param path: sqlite file to connect
        """
        host = kwargs['host']
        port = kwargs['port']
        Logger.info(self.__class__.__name__, 'Kdb+ database client is connecting to %s:%d' % (host, port))
        self.conn = qconnection.QConnection(host=host, port=port)
        self.conn.open()
        if self.conn.is_connected():
            Logger.info(self.__class__.__name__, 'Connection to %s:%d is successful.' % (host, port))    
        else:
            Logger.info(self.__class__.__name__, 'Connection to %s:%d is failed.' % (host, port))    
            
        return self.conn.is_connected()


    def execute(self, sql):
        """
        Execute the sql command
        :param sql: SQL command
        """
        return True

    def commit(self):
        """
        Commit
        """
        return True

    def fetchone(self):
        """
        Fetch one record
        :return Record
        """
        return []

    def fetchall(self):
        """
        Fetch all records
        :return Record
        """
        return []

    def create(self, table, columns, types, primary_key_index=[], is_ifnotexists=True):
        """
        Create table in the database.
        Caveat - Assign the first few column as the keys!!!
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param is_ifnotexists: Create table if not exists keyword
        """
        if len(columns) != len(types):
            raise Exception("Incorrect create statement. Number of columns and that of types are different.\n%s\n%s" % \
                (columns, types))

        if is_ifnotexists:
            ret = self.conn("\\v")
            if ret is not None:
                for t in ret:
                    if table == self.decode_qtypes(t):
                        Logger.info(self.__class__.__name__, "Table %s has been created." % table)
                        return True
                        
            Logger.info(self.__class__.__name__, "Table %s is going to be created." % table)
            
        c = columns[:]
        
        for i in range(0, len(types)):
            t = self.convert_type(types[i])
            if t is str:
                if columns[i].find('date_time') > -1:
                    c[i] += ":`timestamp$()"
                else:
                    c[i] += ":`symbol$()"
            elif t is float:
                c[i] += ":`float$()"
            elif t is int:
                c[i] += ":`int$()"
        
        keys = []
        for i in primary_key_index:
            keys.append(c[i])
        
        for i in sorted(primary_key_index, reverse=True):
            del c[i]
        
        if len(keys) > 0:
            command = '%s:([%s] %s)' % (table, '; '.join(keys), '; '.join(c))
        else:
            command = '%s:(%s)' % (table, '; '.join(c))

        self.lock.acquire()
        try:
            self.conn.sync(command)
        except Exception as e:
            Logger.error(self.__class__.__name__, "Error in creat statement(%s).\n%s" % (command, e))
        finally:
            self.lock.release()
        
        return True

    def insert(self, table, columns, types, values, primary_key_index=[], is_orreplace=False, is_commit=True):
        """
        Insert into the table
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param values: Value array
        :param primary_key_index: An array of indices of primary keys in columns,
                          e.g. [0] means the first column is the primary key
        :param is_orreplace: Indicate if the query is "INSERT OR REPLACE"
        """
        if len(columns) != len(values):
            raise Exception("Incorrect insert statement. Number of columns and that of types are different.\n%s\n%s" % \
                (columns, values))

        v = values[:]
        for i in range(0, len(v)):
            type = self.convert_type(types[i])
            if i in primary_key_index:
                v[i] = "`" + v[i]
            elif type is str:
                if columns[i].find('date_time') > -1:
                    v_len = len(v[i])
                    v[i] = re.sub("(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2}):(\d{2}).(\d*)", \
                                  "\\1.\\2.\\3D\\4:\\5:\\6.\\7", v[i]) + "0" * (27-v_len)
                else:
                    v[i] = "`$\"" + v[i] + "\""
            elif type is float:
                v[i] = float(v[i])
            elif type is int:
                v[i] = int(v[i])

        value_string = '; '.join([str(e) for e in v])
        
        if is_orreplace:
            command = "`%s upsert (%s)" % (table, value_string)
        else:
            command = "`%s insert (%s)" % (table, value_string)

        self.lock.acquire()
        try:
            self.conn.sync(command)
        except Exception as e:
            raise Exception("Error in running insert statement (%s).\n%s" % (command, e))
        finally:
            self.lock.release()

        return True

    def select(self, table, columns=['*'], condition='', orderby='', limit=0, isFetchAll=True):
        """
        Select rows from the table
        :param table: Table name
        :param columns: Selected columns
        :param condition: Where condition
        :param orderby: Order by condition
        :param limit: Rows limit
        :param isFetchAll: Indicator of fetching all
        :return Result rows
        """
        command = ''
        
        # Select columns
        if len(columns) == 1 and columns[0] == '*':
            command = 'select from %s' % table
        else:
            command = 'select %s from %s' % (','.join(columns), table)
        
        # Where condition
        if len(condition) > 0:
            command += ' where %s' % self.parse_condition(condition)
        
        # Order by statement
        if len(orderby) > 0:
            orderbys = [e.strip() for e in orderby.split(',')]
            for i in range(0, len(orderbys)):
                ep = orderbys[i].split(' ')
                if len(ep) == 1:
                    orderbys[i] = "`%s xasc " % ep[0]
                elif len(ep) == 2 and ep[1] == "asc":
                    orderbys[i] = "`%s xasc " % ep[0]
                elif len(ep) == 2 and ep[1] == "desc":
                    orderbys[i] = "%s xdesc " % ep[0]
                else:
                    raise Exception("Incorrect orderby (%s) statement in select command." % orderby)
               
            command = ''.join(orderbys) + command
            command = '`' + command
        
        # Limit
        if limit > 0:
            command = "%d#%s" % (limit, command)

        self.lock.acquire()
        try:
            select_ret = self.conn(command)
            select_ret = select_ret[:]
        except Exception as e:
            raise Exception("Error in running select statement (%s).\n%s" % (command, e))
        finally:
            self.lock.release()       
        
        ret = []
        if isinstance(select_ret, QKeyedTable):
            for key, value in select_ret.iteritems():
                row = list(key) + list(value)
                row = [self.decode_qtypes(e) for e in row]
                ret.append(row)
        elif isinstance(select_ret, QTable):
            # Empty records
            if select_ret[0][0] == -2147483648:
                pass
            else:
                for value in select_ret:
                    if len(value) == 0 or \
                       (isinstance(value[0], list) and len(value[0]) == 0):
                        pass
                    else:
                        row = []
                        for e in value:
                            row.append(self.decode_qtypes(e))
                            
                        ret.append(row)
        elif isinstance(select_ret, QList):
            for e in select_ret:
                ret.append(self.decode_qtypes(e))
        elif select_ret is None:
            # Return empty list
            pass
        else:
            raise Exception("Unknown type (%s) in kdb client select statement.\n%s" % (type(select_ret), select_ret))


        return ret

    def delete(self, table, condition='1==1'):
        """
        Delete rows from the table
        :param table: Table name
        :param condition: Where condition
        """
        if condition == '1==1':
            statement = 'delete from `%s' % (table)
        else:
            statement = 'delete from `%s where %s' % (table, self.parse_condition(condition))
        self.conn.sync(statement)
        return True

if __name__ == '__main__':
    Logger.init_log()
    db_client = KdbPlusClient()
    db_client.connect(host='localhost', port=5000)
    db_client.create('test', ['c1', 'c2', 'c3', 'c4'], ['varchar(20)', 'int', 'decimal(8, 20)', 'int'], [0], False)
    db_client.insert('test', ['c1', 'c2', 'c3', 'c4'], ['abc', 1, 1.1, 5])
    db_client.insert('test', ['c1', 'c2', 'c3', 'c4'], ['efg', 2, 2.2, 6])
    db_client.insert('test', ['c1', 'c2', 'c3', 'c4'], ['hij', 3, 3.3, 7])
    # Logger.info('test', db_client.select('test', columns=['*']))
    Logger.info('test', db_client.select('test', columns=['c2', 'c3'], condition='c1 >= "abc" and c2 > 1'))
    # Logger.info('test', db_client.select('test', columns=['*'], orderby='c1 desc', limit=1))
    db_client.delete('test', 'c1="abc"')
    


from befh.api_socket import ApiSocket
from befh.util import Logger
import websocket
import threading
import json
import time

class WebSocketApiClient(ApiSocket):
    """
    Generic REST API call
    """
    def __init__(self, id):
        """
        Constructor
        :param id: Socket id
        """
        ApiSocket.__init__(self)
        self.ws = None              # Web socket
        self.id = id
        self.wst = None             # Web socket thread
        self._connecting = False
        self._connected = False
        self.on_message_handlers = []
        self.on_open_handlers = []
        self.on_close_handlers = []
        self.on_error_handlers = []

    def connect(self, url,
                on_message_handler=None,
                on_open_handler=None,
                on_close_handler=None,
                on_error_handler=None,
                reconnect_interval=10):
        """
        :param url: Url link
        :param on_message_handler: Message handler which take the message as
                           the first argument
        :param on_open_handler: Socket open handler which take the socket as
                           the first argument
        :param on_close_handler: Socket close handler which take the socket as
                           the first argument
        :param on_error_handler: Socket error handler which take the socket as
                           the first argument and the error as the second
                           argument
        :param reconnect_interval: The time interval for reconnection
        """
        Logger.info(self.__class__.__name__, "Connecting to socket <%s>..." % self.id)
        if on_message_handler is not None:
            self.on_message_handlers.append(on_message_handler)
        if on_open_handler is not None:
            self.on_open_handlers.append(on_open_handler)
        if on_close_handler is not None:
            self.on_close_handlers.append(on_close_handler)
        if on_error_handler is not None:
            self.on_error_handlers.append(on_error_handler)

        if not self._connecting and not self._connected:
            self._connecting = True
            self.ws = websocket.WebSocketApp(url,
                                             on_message=self.__on_message,
                                             on_close=self.__on_close,
                                             on_open=self.__on_open,
                                             on_error=self.__on_error)
            self.wst = threading.Thread(target=lambda: self.__start(reconnect_interval=reconnect_interval))
            self.wst.start()

        return self.wst

    def send(self, msg):
        """
        Send message
        :param msg: Message
        :return:
        """
        self.ws.send(msg)

    def __start(self, reconnect_interval=10):
        while True:
            self.ws.run_forever()
            Logger.info(self.__class__.__name__, "Socket <%s> is going to reconnect..." % self.id)
            time.sleep(reconnect_interval)

    def __on_message(self, ws, m):
        m = json.loads(m)
        if len(self.on_message_handlers) > 0:
            for handler in self.on_message_handlers:
                handler(m)

    def __on_open(self, ws):
        Logger.info(self.__class__.__name__, "Socket <%s> is opened." % self.id)
        self._connected = True
        if len(self.on_open_handlers) > 0:
            for handler in self.on_open_handlers:
                handler(ws)
        
    def __on_close(self, ws):
        Logger.info(self.__class__.__name__, "Socket <%s> is closed." % self.id)
        self._connecting = False
        self._connected = False
        if len(self.on_close_handlers) > 0:
            for handler in self.on_close_handlers:
                handler(ws)
        
    def __on_error(self, ws, error):
        Logger.info(self.__class__.__name__, "Socket <%s> error:\n %s" % (self.id, error))
        if len(self.on_error_handlers) > 0:
            for handler in self.on_error_handlers:
                handler(ws, error)

if __name__ == '__main__':
    Logger.init_log()
    socket = WebSocketApiClient('test')
    socket.connect('ws://localhost', reconnect_interval=1)
    time.sleep(10)


#!/bin/python

class ApiSocket:
    """
    API socket
    """
    def __init__(self):
        pass
    
    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        return None

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        return None
    
    def get_order_book(self, instmt):
        """
        Get order book
        :param instmt: Instrument
        :return: Object L2Depth
        """
        return None

    def get_trades(self, instmt, trade_id):
        """
        Get trades
        :param instmt: Instrument
        :param trade_id: Trade id
        :return: List of trades
        """
        return None



from datetime import datetime
import logging


class Logger:
    logger = None

    @staticmethod
    def init_log(output=None):
        """
        Initialise the logger
        """
        Logger.logger = logging.getLogger('BitcoinExchangeFH')
        Logger.logger.setLevel(logging.ERROR)
        Logger.logger.setLevel(logging.INFO)
        formatter = logging.Formatter('%(asctime)s - %(levelname)s \n%(message)s\n')
        if output is None:
            slogger = logging.StreamHandler()
            slogger.setFormatter(formatter)
            Logger.logger.addHandler(slogger)
        else:
            flogger = logging.FileHandler(output)
            flogger.setFormatter(formatter)
            Logger.logger.addHandler(flogger)

    @staticmethod
    def info(method, str):
        """
        Write info log
        :param method: Method name
        :param str: Log message
        """
        Logger.logger.info('[%s]\n%s\n' % (method, str))

    @staticmethod
    def error(method, str):
        """
        Write info log
        :param method: Method name
        :param str: Log message
        """
        Logger.logger.error('[%s]\n%s\n' % (method, str))



class DatabaseClient:
    """
    Base database client
    """
    def __init__(self):
        """
        Constructor
        """
        pass

    @classmethod
    def convert_str(cls, val):
        """
        Convert the value to string
        :param val: Can be string, int or float
        :return:
        """
        if isinstance(val, str):
            return "'" + val + "'"
        elif isinstance(val, bytes):
            return "'" + str(val) + "'"
        elif isinstance(val, int):
            return str(val)
        elif isinstance(val, float):
            return "%.8f" % val
        else:
            raise Exception("Cannot convert value (%s)<%s> to string. Value is not a string, an integer nor a float" %\
                            (val, type(val)))

    def connect(self, **args):
        """
        Connect
        :return True if it is connected
        """
        return True

    def create(self, table, columns, types, primary_key_index=[], is_ifnotexists=True):
        """
        Create table in the database
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param primary_key_index: An array of indices of primary keys in columns, 
                                  e.g. [0] means the first column is the primary key
        :param is_ifnotexists: Create table if not exists keyword
        """
        return True

    def insert(self, table, columns, types, values, primary_key_index=[], is_orreplace=False, is_commit=True):
        """
        Insert into the table
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param values: Value array
        :param primary_key_index: An array of indices of primary keys in columns,
                                  e.g. [0] means the first column is the primary key
        :param is_orreplace: Indicate if the query is "INSERT OR REPLACE"
        :param is_commit: Indicate if the query is committed (in sql command database mostly)
        """
        return True

    def select(self, table, columns=['*'], condition='', orderby='', limit=0, isFetchAll=True):
        """
        Select rows from the table
        :param table: Table name
        :param columns: Selected columns
        :param condition: Where condition
        :param orderby: Order by condition
        :param limit: Rows limit
        :param isFetchAll: Indicator of fetching all
        :return Result rows
        """

    def close(self):
        """
        Close connection
        :return:
        """
        return True



#!/bin/python

from befh.sql_client import SqlClient
import sqlite3

class SqliteClient(SqlClient):
    """
    Sqlite client
    """
    @classmethod
    def replace_keyword(cls):
        return 'insert or replace into'

    def __init__(self):
        """
        Constructor
        """
        SqlClient.__init__(self)

    def connect(self, **kwargs):
        """
        Connect
        :param path: sqlite file to connect
        """
        path = kwargs['path']
        self.conn = sqlite3.connect(path, check_same_thread=False)
        self.cursor = self.conn.cursor()
        return self.conn is not None and self.cursor is not None
        
    def execute(self, sql):
        """
        Execute the sql command
        :param sql: SQL command
        """
        self.cursor.execute(sql)
        
    def commit(self):
        """
        Commit
        """    
        self.conn.commit()
        
    def fetchone(self):
        """
        Fetch one record
        :return Record
        """        
        return self.cursor.fetchone()     

    def fetchall(self):
        """
        Fetch all records
        :return Record
        """        
        return self.cursor.fetchall()




import json

class Instrument(object):
    def __init__(self,
                 exchange_name,
                 instmt_name,
                 instmt_code,
                 **param):
        """
        Constructor
        :param exchange: Exchange name
        :param instmt_code: Instrument code
        :param param: Options parameters, e.g. restful_order_book_link
        :return:
        """
        self.exchange_name = exchange_name
        self.instmt_name = instmt_name
        self.instmt_code = instmt_code
        self.order_book_table_name = ''
        self.trades_table_name = ''
        self.order_book_id = 0
        self.trade_id = 0
        self.exch_trade_id = '0'
        self.subscribed = False
        self.recovered = True
        self.l2_depth = None
        self.prev_l2_depth = None
        self.last_trade = None
        self.order_book_channel_id = ''
        self.trades_channel_id = ''

    def copy(self, obj):
        """
        Copy constructor
        """
        self.exchange_name = obj.exchange_name
        self.instmt_name = obj.instmt_name
        self.instmt_code = obj.instmt_code
        self.order_book_table_name = obj.order_book_table_name
        self.trades_table_name = obj.trades_table_name
        self.order_book_id = obj.order_book_id
        self.trade_id = obj.trade_id
        self.exch_trade_id = obj.exch_trade_id
        self.subscribed = obj.subscribed
        self.recovered = obj.recovered
        self.l2_depth = obj.l2_depth
        self.prev_l2_depth = obj.prev_l2_depth
        self.last_trade = obj.last_trade
        self.order_book_channel_id = obj.order_book_channel_id
        self.trades_channel_id = obj.trades_channel_id

    def get_exchange_name(self):
        return self.exchange_name
        
    def get_instmt_name(self):
        return self.instmt_name

    def get_instmt_code(self):
        return self.instmt_code

    def get_order_book_table_name(self):
        return self.order_book_table_name

    def get_trades_table_name(self):
        return self.trades_table_name

    def get_order_book_id(self):
        return self.order_book_id

    def get_trade_id(self):
        return self.trade_id

    def get_exch_trade_id(self):
        return self.exch_trade_id

    def get_subscribed(self):
        return self.subscribed

    def get_recovered(self):
        return self.recovered

    def get_l2_depth(self):
        return self.l2_depth

    def get_prev_l2_depth(self):
        return self.prev_l2_depth

    def get_last_trade(self):
        return self.last_trade

    def get_order_book_channel_id(self):
        return self.order_book_channel_id

    def get_trades_channel_id(self):
        return self.trades_channel_id

    def set_trade_id(self, trade_id):
        self.trade_id = trade_id
        
    def set_order_book_table_name(self, order_book_table_name):
        self.order_book_table_name = order_book_table_name
        
    def set_trades_table_name(self, trades_table_name):
        self.trades_table_name = trades_table_name
        
    def set_trades_channel_id(self, trades_channel_id):
        self.trades_channel_id = trades_channel_id

    def set_order_book_id(self, order_book_id):
        self.order_book_id = order_book_id

    def set_exch_trade_id(self, exch_trade_id):
        assert isinstance(exch_trade_id, str), \
            "exch_trade_id (%s) = %s" % (type(exch_trade_id), exch_trade_id)
        self.exch_trade_id = exch_trade_id

    def set_subscribed(self, subscribed):
        self.subscribed = subscribed

    def set_recovered(self, recovered):
        self.recovered = recovered
        
    def set_l2_depth(self, l2_depth):
        self.l2_depth = l2_depth
        
    def set_prev_l2_depth(self, prev_l2_depth):
        self.prev_l2_depth = prev_l2_depth

    def set_last_trade(self, trade):
        self.last_trade = trade

    def set_order_book_channel_id(self, order_book_channel_id):
        self.order_book_channel_id = order_book_channel_id
        
    def incr_order_book_id(self):
        assert isinstance(self.order_book_id, int), "Type(self.order_book_id) = %s" % type(self.order_book_id)
        self.order_book_id += 1

    def incr_trade_id(self):
        assert isinstance(self.trade_id, int), "Type(self.trade_id) = %s" % type(self.trade_id)
        self.trade_id += 1        
        
        


from befh.instrument import Instrument
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

class SubscriptionManager:
    def __init__(self, config_path):
        """
        Constructor
        """
        self.config = ConfigParser.ConfigParser()
        self.config.read(config_path)
        
    def get_instmt_ids(self):
        """
        Return all the instrument ids
        """
        return self.config.sections()
    
    def get_instrument(self, instmt_id):
        """
        Return the instrument object by instrument id
        :param instmt_id: Instrument ID
        :return Instrument object
        """
        exchange_name = self.config.get(instmt_id, 'exchange')
        instmt_name = self.config.get(instmt_id, 'instmt_name')
        instmt_code = self.config.get(instmt_id, 'instmt_code')
        enabled = int(self.config.get(instmt_id, 'enabled'))
        params = dict(self.config.items(instmt_id))
        del params['exchange']
        del params['instmt_name']
        del params['instmt_code']
        del params['enabled']
        
        if enabled == 1:
            return Instrument(exchange_name, instmt_name, instmt_code, **params)
        else:
            return None
        
    def get_subscriptions(self):
        """
        Get all the subscriptions from the configuration file
        :return List of instrument objects
        """
        instmts = [self.get_instrument(inst) for inst in self.get_instmt_ids()]
        return [instmt for instmt in instmts if instmt is not None]
        



from befh.database_client import DatabaseClient
from befh.util import Logger
import threading
import re
import zmq
import time

class ZmqClient(DatabaseClient):
    """
    Zmq Client
    """
    def __init__(self):
        """
        Constructor
        """
        DatabaseClient.__init__(self)
        self.context = zmq.Context()
        self.conn = self.context.socket(zmq.PUB)
        self.lock = threading.Lock()

    def connect(self, **kwargs):
        """
        Connect
        :param path: sqlite file to connect
        """
        addr = kwargs['addr']
        Logger.info(self.__class__.__name__, 'Zmq client is connecting to %s' % addr)
        self.conn.bind(addr)
        return self.conn is not None


    def execute(self, sql):
        """
        Execute the sql command
        :param sql: SQL command
        """
        return True

    def commit(self):
        """
        Commit
        """
        return True

    def fetchone(self):
        """
        Fetch one record
        :return Record
        """
        return []

    def fetchall(self):
        """
        Fetch all records
        :return Record
        """
        return []

    def create(self, table, columns, types, primary_key_index=[], is_ifnotexists=True):
        """
        Create table in the database.
        Caveat - Assign the first few column as the keys!!!
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param is_ifnotexists: Create table if not exists keyword
        """
        return True

    def insert(self, table, columns, types, values, primary_key_index=[], is_orreplace=False, is_commit=True):
        """
        Insert into the table
        :param table: Table name
        :param columns: Column array
        :param types: Type array
        :param values: Value array
        :param primary_key_index: An array of indices of primary keys in columns,
                          e.g. [0] means the first column is the primary key
        :param is_orreplace: Indicate if the query is "INSERT OR REPLACE"
        """
        ret = dict(zip(columns, values))
        ret['table'] = table
        self.lock.acquire()
        self.conn.send_pyobj(ret)
        self.lock.release()
        return True

    def select(self, table, columns=['*'], condition='', orderby='', limit=0, isFetchAll=True):
        """
        Select rows from the table
        :param table: Table name
        :param columns: Selected columns
        :param condition: Where condition
        :param orderby: Order by condition
        :param limit: Rows limit
        :param isFetchAll: Indicator of fetching all
        :return Result rows
        """
        return []

    def delete(self, table, condition='1==1'):
        """
        Delete rows from the table
        :param table: Table name
        :param condition: Where condition
        """
        return True

if __name__ == '__main__':
    Logger.init_log()
    db_client = ZmqClient()
    db_client.connect(addr='ipc://test')
    for i in range(1, 100):
        db_client.insert('test', ['c1', 'c2', 'c3', 'c4'], [], ['abc', i, 1.1, 5])
        time.sleep(1)
    


from befh.market_data import L2Depth, Trade
from befh.exchange import ExchangeGateway
from befh.instrument import Instrument
from befh.ws_api_socket import WebSocketApiClient
from befh.util import Logger
import time
import threading
import json
from functools import partial
from datetime import datetime


class ExchGwBitfinexWs(WebSocketApiClient):
    """
    Exchange gateway BTCC RESTfulApi
    """
    def __init__(self):
        """
        Constructor
        """
        WebSocketApiClient.__init__(self, 'ExchGwBitfinex')
            
    @classmethod
    def get_link(cls):
        return 'wss://api2.bitfinex.com:3000/ws'

    @classmethod
    def get_order_book_subscription_string(cls, instmt):
        return json.dumps({"event":"subscribe", "channel": "book", "pair": instmt.get_instmt_code(), "freq": "F0"})

    @classmethod
    def get_trades_subscription_string(cls, instmt):
        return json.dumps({"event":"subscribe", "channel": "trades", "pair": instmt.get_instmt_code()})

    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        # No order book mapping from config. Need to decode here.
        l2_depth = instmt.get_l2_depth()
        l2_depth.date_time = datetime.utcnow().strftime("%Y%m%d %H:%M:%S.%f")
        if isinstance(raw[0], list):
            # Start subscription
            for i in range(0, 25):
                bid = raw[i]
                ask = raw[25+i]

                l2_depth.bids[i] = L2Depth.Depth(price=bid[0], count=bid[1], volume=bid[2])
                l2_depth.asks[i] = L2Depth.Depth(price=ask[0], count=ask[1], volume=-ask[2])

        else:
            price = raw[1]
            count = raw[2]
            volume = raw[3]
            found = False

            if count == 0:
                # Deletion
                if volume > 0:
                    for i in range(0, len(l2_depth.bids)):
                        if price == l2_depth.bids[i].price:
                            found = True
                            del l2_depth.bids[i]
                            break
                else:
                    for i in range(0, len(l2_depth.asks)):
                        if price == l2_depth.asks[i].price:
                            found = True
                            del l2_depth.asks[i]
                            break

                if not found:
                    depth_text = ""
                    for i in range(0, l2_depth.depth):
                        if i < len(l2_depth.bids):
                            depth_text += "%.4f,%d,%.4f" % \
                              (l2_depth.bids[i].volume, \
                               l2_depth.bids[i].count, \
                               l2_depth.bids[i].price)
                        else:
                            depth_text += "                   "
                        depth_text += "<--->"
                        if i < len(l2_depth.asks):
                            depth_text += "%.4f,%d,%.4f" % \
                                          (l2_depth.asks[i].volume, \
                                           l2_depth.asks[i].count, \
                                           l2_depth.asks[i].price)
                        else:
                            depth_text += "                   "
                        depth_text += "\n"
                    Logger.info(cls.__name__, "Cannot find the deletion of the message: %s\nDepth:\n%s\n" % \
                              (raw, depth_text))
            else:
                # Insertion/Update
                if volume > 0:
                    # Update
                    for i in range(0, len(l2_depth.bids)):
                        if price == l2_depth.bids[i].price:
                            l2_depth.bids[i].count = count
                            l2_depth.bids[i].volume = volume
                            found = True
                            break

                    if not found:
                        # Insertion
                        l2_depth.bids.append(L2Depth.Depth(price=price,
                                                         count=count,
                                                         volume=volume))
                        l2_depth.sort_bids()

                        if len(l2_depth.bids) > l2_depth.depth * 2:
                            del l2_depth.bids[l2_depth.depth:]
                else:
                    for i in range(0, len(l2_depth.asks)):
                        # Update
                        if price == l2_depth.asks[i].price:
                            l2_depth.asks[i].count = count
                            l2_depth.asks[i].volume = -volume
                            found = True
                            break

                    if not found:
                        # Insertion
                        l2_depth.asks.append(L2Depth.Depth(price=price,
                                                    count=count,
                                                    volume=-volume))
                        l2_depth.sort_asks()

                        if len(l2_depth.asks) > l2_depth.depth * 2:
                            del l2_depth.asks[l2_depth.depth:]

        return l2_depth

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        trade = Trade()
        trade_id = raw[0]
        timestamp = raw[1]
        trade_price = raw[2]
        trade_volume = raw[3]
        
        trade.date_time = datetime.utcfromtimestamp(timestamp).strftime("%Y%m%d %H:%M:%S.%f")
        trade.trade_side = Trade.Side.BUY if trade_volume > 0 else Trade.Side.SELL
        trade.trade_volume = abs(trade_volume)
        trade.trade_id = str(trade_id)
        trade.trade_price = trade_price

        return trade


class ExchGwBitfinex(ExchangeGateway):
    """
    Exchange gateway BTCC
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwBitfinexWs(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'Bitfinex'

    def on_open_handler(self, instmt, ws):
        """
        Socket on open handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is subscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        if not instmt.get_subscribed():
            ws.send(self.api_socket.get_order_book_subscription_string(instmt))
            ws.send(self.api_socket.get_trades_subscription_string(instmt))
            instmt.set_subscribed(True)

    def on_close_handler(self, instmt, ws):
        """
        Socket on close handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is unsubscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        instmt.set_subscribed(False)

    def on_message_handler(self, instmt, message):
        """
        Incoming message handler
        :param instmt: Instrument
        :param message: Message
        """
        if isinstance(message, dict):
            keys = message.keys()
            if 'event' in keys and message['event'] == 'info' and  'version' in keys:
                Logger.info(self.__class__.__name__, "Bitfinex version: %s" % message['version'])
            elif 'event' in keys and message['event'] == 'subscribed':
                if instmt.get_instmt_code() == message['pair']:
                    if message['channel'] == 'book':
                        instmt.set_order_book_channel_id(message['chanId'])
                    elif message['channel'] == 'trades':
                        instmt.set_trades_channel_id(message['chanId'])
                    else:
                        raise Exception("Unknown channel %s : <%s>" % (message['channel'], message))

                    Logger.info(self.__class__.__name__, 'Subscription: %s, pair: %s, channel Id: %s' % \
                              (message['channel'], instmt.get_instmt_code(), message['chanId']))
        elif isinstance(message, list):
            if message[0] == instmt.get_order_book_channel_id():
                if isinstance(message[1], list):
                    self.api_socket.parse_l2_depth(instmt, message[1])
                elif len(message) != 2:
                    instmt.set_prev_l2_depth(instmt.get_l2_depth().copy())
                    self.api_socket.parse_l2_depth(instmt, message)
                else:
                    return

                if instmt.get_l2_depth().is_diff(instmt.get_prev_l2_depth()):
                    instmt.incr_order_book_id()
                    self.insert_order_book(instmt)

            elif message[0] == instmt.get_trades_channel_id():
                # No recovery trade
                
                # if isinstance(message[1], list):
                #     raw_trades = message[1]
                #     raw_trades.sort(key=lambda x:x[0])
                #     for raw in raw_trades:
                #         trade = self.api_socket.parse_trade(instmt, raw)
                #         try:
                #             if int(trade.trade_id) > int(instmt.get_exch_trade_id()):
                #                 instmt.incr_trade_id()
                #                 instmt.set_exch_trade_id(trade.trade_id)
                #                 self.insert_trade(instmt, trade)
                #         except Exception as e:
                #             Logger.info('test', "trade.trade_id(%s):%s" % (type(trade.trade_id), trade.trade_id))
                #             Logger.info('test', "instmt.get_exch_trade_id()(%s):%s" % (type(instmt.get_exch_trade_id()), instmt.get_exch_trade_id()))
                #             raise e                            

                if message[1] == 'tu':
                    trade = self.api_socket.parse_trade(instmt, message[3:])
                    if int(trade.trade_id) > int(instmt.get_exch_trade_id()):
                        instmt.incr_trade_id()
                        instmt.set_exch_trade_id(trade.trade_id)
                        self.insert_trade(instmt, trade)

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_l2_depth(L2Depth(25))
        instmt.set_prev_l2_depth(L2Depth(25))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                       instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                               instmt.get_instmt_name()))
        instmt.set_order_book_id(self.get_order_book_init(instmt))
        trade_id, last_exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(last_exch_trade_id)
        return [self.api_socket.connect(self.api_socket.get_link(),
                                        on_message_handler=partial(self.on_message_handler, instmt),
                                        on_open_handler=partial(self.on_open_handler, instmt),
                                        on_close_handler=partial(self.on_close_handler, instmt))]




from befh.restful_api_socket import RESTfulApiSocket
from befh.exchange import ExchangeGateway
from befh.market_data import L2Depth, Trade
from befh.util import Logger
from befh.instrument import Instrument
from befh.sql_client_template import SqlClientTemplate
import time
import threading
from functools import partial
from datetime import datetime


class ExchGwApiGatecoin(RESTfulApiSocket):
    """
    Exchange gateway RESTfulApi
    """
    def __init__(self):
        RESTfulApiSocket.__init__(self)
        
    @classmethod
    def get_trade_timestamp_field_name(cls):
        return 'transactionTime'
    
    @classmethod
    def get_bids_field_name(cls):
        return 'bids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'asks'
        
    @classmethod
    def get_order_book_price_field_name(cls):
        return 'price'
        
    @classmethod
    def get_order_book_volume_field_name(cls):
        return 'volume'
        
    @classmethod
    def get_trade_side_field_name(cls):
        return 'way'
        
    @classmethod
    def get_trade_id_field_name(cls):
        return 'transactionId'
        
    @classmethod
    def get_trade_price_field_name(cls):
        return 'price'        
        
    @classmethod
    def get_trade_volume_field_name(cls):
        return 'quantity'        
        
    @classmethod
    def get_order_book_link(cls, instmt):
        return "https://api.gatecoin.com/Public/MarketDepth/%s" % instmt.get_instmt_code()

    @classmethod
    def get_trades_link(cls, instmt):
        if int(instmt.get_exch_trade_id()) > 0:
            return "https://api.gatecoin.com/Public/Transactions/%s?since=%s" % \
                (instmt.get_instmt_code(), instmt.get_exch_trade_id())
        else:
            return "https://api.gatecoin.com/Public/Transactions/%s" % \
                (instmt.get_instmt_code())         
                
    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        keys = list(raw.keys())
        if cls.get_bids_field_name() in keys and \
          cls.get_asks_field_name() in keys:
            
            l2_depth = L2Depth()
            # Bids
            bids = raw[cls.get_bids_field_name()]
            bid_level = -1
            for bid in bids:
                price = bid[cls.get_order_book_price_field_name()]
                volume = bid[cls.get_order_book_volume_field_name()]
                
                if bid_level == -1 or l2_depth.bids[bid_level].price != price:
                    bid_level += 1
                    
                    if bid_level < 5:
                        l2_depth.bids[bid_level].price = float(price)
                    else:
                        break
                
                l2_depth.bids[bid_level].volume += float(volume)
            
            # Asks
            asks = raw[cls.get_asks_field_name()]
            ask_level = -1
            for ask in asks:
                price = ask[cls.get_order_book_price_field_name()]
                volume = ask[cls.get_order_book_volume_field_name()]
                
                if ask_level == -1 or l2_depth.asks[ask_level].price != price:
                    ask_level += 1
                    
                    if ask_level < 5:
                        l2_depth.asks[ask_level].price = float(price)
                    else:
                        break
                
                l2_depth.asks[ask_level].volume += float(volume)
            
            return l2_depth
            
        else:
            raise Exception('Does not contain order book keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        trade = Trade()
        keys = list(raw.keys())
        
        if cls.get_trade_timestamp_field_name() in keys and \
          cls.get_trade_id_field_name() in keys and \
          cls.get_trade_price_field_name() in keys and \
          cls.get_trade_volume_field_name() in keys:
        
            # Date time
            date_time = float(raw[cls.get_trade_timestamp_field_name()])
            trade.date_time = datetime.utcfromtimestamp(date_time).strftime("%Y%m%d %H:%M:%S.%f")      
            
            # Trade side
            trade.trade_side = 1
                
            # Trade id
            trade.trade_id = str(raw[cls.get_trade_id_field_name()])
            
            # Trade price
            trade.trade_price = float(str(raw[cls.get_trade_price_field_name()]))
            
            # Trade volume
            trade.trade_volume = float(str(raw[cls.get_trade_volume_field_name()]))
        else:
            raise Exception('Does not contain trade keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))        

        return trade

    @classmethod
    def get_order_book(cls, instmt):
        """
        Get order book
        :param instmt: Instrument
        :return: Object L2Depth
        """
        res = cls.request(cls.get_order_book_link(instmt))
        if len(res) > 0:
            return cls.parse_l2_depth(instmt=instmt,
                                       raw=res)
        else:
            return None

    @classmethod
    def get_trades(cls, instmt):
        """
        Get trades
        :param instmt: Instrument
        :param trade_id: Trade id
        :return: List of trades
        """
        link = cls.get_trades_link(instmt)
        res = cls.request(link)
        trades = []
        if 'transactions' in res.keys():
            trades_raw = res['transactions']
            if len(trades_raw) > 0:
                for t in trades_raw:
                    trade = cls.parse_trade(instmt=instmt,
                                             raw=t)
                    trades.append(trade)

        return trades


class ExchGwGatecoin(ExchangeGateway):
    """
    Exchange gateway
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwApiGatecoin(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'Gatecoin'

    def get_order_book_worker(self, instmt):
        """
        Get order book worker
        :param instmt: Instrument
        """
        instmt.set_order_book_id(self.get_order_book_init(instmt))

        while True:
            try:
                l2_depth = self.api_socket.get_order_book(instmt)
                if l2_depth is not None and l2_depth.is_diff(instmt.get_l2_depth()):
                    instmt.set_prev_l2_depth(instmt.get_l2_depth())
                    instmt.set_l2_depth(l2_depth)
                    instmt.incr_order_book_id()
                    self.insert_order_book(instmt)
            except Exception as e:
                Logger.error(self.__class__.__name__, "Error in order book: %s" % e)
            time.sleep(1)

    def get_trades_worker(self, instmt):
        """
        Get order book worker thread
        :param instmt: Instrument name
        """
        trade_id, exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(exch_trade_id)

        while True:
            try:
                ret = self.api_socket.get_trades(instmt)
                if ret is None or len(ret) == 0:
                    time.sleep(1)
                    continue
            except Exception as e:
                Logger.error(self.__class__.__name__, "Error in trades: %s" % e)                
                
            for trade in ret:
                assert isinstance(trade.trade_id, str), "trade.trade_id(%s) = %s" % (type(trade.trade_id), trade.trade_id)
                assert isinstance(instmt.get_exch_trade_id(), str), \
                       "instmt.get_exch_trade_id()(%s) = %s" % (type(instmt.get_exch_trade_id()), instmt.get_exch_trade_id())
                if int(trade.trade_id) > int(instmt.get_exch_trade_id()):
                    instmt.set_exch_trade_id(trade.trade_id)
                    instmt.incr_trade_id()
                    self.insert_trade(instmt, trade)
            
            # After the first time of getting the trade, indicate the instrument
            # is recovered
            if not instmt.get_recovered():
                instmt.set_recovered(True)

            time.sleep(1)

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_l2_depth(L2Depth(5))
        instmt.set_prev_l2_depth(L2Depth(5))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                        instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                                instmt.get_instmt_name()))
        instmt.set_recovered(False)
        t1 = threading.Thread(target=partial(self.get_order_book_worker, instmt))
        t1.start()
        t2 = threading.Thread(target=partial(self.get_trades_worker, instmt))
        t2.start()
        return [t1, t2]
        
        
if __name__ == '__main__':
    Logger.init_log()
    exchange_name = 'Gatecoin'
    instmt_name = 'BTCHKD'
    instmt_code = 'BTCHKD'
    instmt = Instrument(exchange_name, instmt_name, instmt_code)    
    db_client = SqlClientTemplate()
    exch = ExchGwGatecoin(db_client)
    instmt.set_l2_depth(L2Depth(5))
    instmt.set_prev_l2_depth(L2Depth(5))
    instmt.set_order_book_table_name(exch.get_order_book_table_name(instmt.get_exchange_name(),
                                                                    instmt.get_instmt_name()))
    instmt.set_trades_table_name(exch.get_trades_table_name(instmt.get_exchange_name(),
                                                            instmt.get_instmt_name()))
    instmt.set_recovered(False)    
    # exch.get_order_book_worker(instmt)
    exch.get_trades_worker(instmt)


from befh.restful_api_socket import RESTfulApiSocket
from befh.exchange import ExchangeGateway
from befh.market_data import L2Depth, Trade
from befh.instrument import Instrument
from befh.util import Logger
import time
import threading
from functools import partial
from datetime import datetime


class ExchGwKrakenRestfulApi(RESTfulApiSocket):
    """
    Exchange socket
    """
    def __init__(self):
        RESTfulApiSocket.__init__(self)

    @classmethod
    def get_bids_field_name(cls):
        return 'bids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'asks'
        
    @classmethod
    def get_order_book_link(cls, instmt):
        return 'https://api.kraken.com/0/public/Depth?pair=%s&count=5' % instmt.get_instmt_code()
        
    @classmethod
    def get_trades_link(cls, instmt):
        if instmt.get_exch_trade_id() != '':
            return 'https://api.kraken.com/0/public/Trades?pair=%s&since=%s' % \
                (instmt.get_instmt_code(), instmt.get_exch_trade_id())
        else:
            return 'https://api.kraken.com/0/public/Trades?pair=%s' % instmt.get_instmt_code()

    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        l2_depth = L2Depth()
        keys = list(raw.keys())
        if cls.get_bids_field_name() in keys and \
           cls.get_asks_field_name() in keys:
            # Bids
            bids = raw[cls.get_bids_field_name()]
            bids = sorted(bids, key=lambda x: x[0], reverse=True)
            for i in range(0, len(bids)):
                l2_depth.bids[i].price = float(bids[i][0]) if type(bids[i][0]) != float else bids[i][0]
                l2_depth.bids[i].volume = float(bids[i][1]) if type(bids[i][1]) != float else bids[i][1]
                
            # Asks
            asks = raw[cls.get_asks_field_name()]
            asks = sorted(asks, key=lambda x: x[0])
            for i in range(0, len(asks)):
                l2_depth.asks[i].price = float(asks[i][0]) if type(asks[i][0]) != float else asks[i][0]
                l2_depth.asks[i].volume = float(asks[i][1]) if type(asks[i][1]) != float else asks[i][1]                    

        return l2_depth

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        trade = Trade()
        
        # Trade price
        trade.trade_price = float(str(raw[0]))
        
        # Trade volume
        trade.trade_volume = float(str(raw[1]))
        
        # Timestamp
        date_time = float(raw[2])
        trade.date_time = datetime.utcfromtimestamp(date_time).strftime("%Y%m%d %H:%M:%S.%f")
        
        # Trade side
        trade.trade_side = Trade.parse_side(raw[3])

        # Trade id
        trade.trade_id = trade.date_time + '-' + str(instmt.get_exch_trade_id())

        return trade

    @classmethod
    def get_order_book(cls, instmt):
        """
        Get order book
        :param instmt: Instrument
        :return: Object L2Depth
        """
        res = cls.request(cls.get_order_book_link(instmt))
        if len(res) > 0 and 'error' in res and len(res['error']) == 0:
            res = list(res['result'].values())[0]
            return cls.parse_l2_depth(instmt=instmt,
                                       raw=res)
        else:
            Logger.error(cls.__name__, "Cannot parse the order book. Return:\n%s" % res)
            return None

    @classmethod
    def get_trades(cls, instmt):
        """
        Get trades
        :param instmt: Instrument
        :param trade_id: Trade id
        :return: List of trades
        """
        res = cls.request(cls.get_trades_link(instmt))

        trades = []
        if len(res) > 0 and 'error' in res and len(res['error']) == 0:
            res = res['result']
            if 'last' in res.keys():
                instmt.set_exch_trade_id(res['last'])
                del res['last']

            res = list(res.values())[0]

            for t in res:
                trade = cls.parse_trade(instmt=instmt,
                                        raw=t)
                trades.append(trade)

        return trades


class ExchGwKraken(ExchangeGateway):
    """
    Exchange gateway
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwKrakenRestfulApi(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'Kraken'

    def get_trades_init(self, instmt):
        """
        Initialization method in get_trades
        :param instmt: Instrument
        :return: Last id
        """
        if self.data_mode & ExchangeGateway.DataMode.TRADES_ONLY:
            table_name = self.get_trades_table_name(instmt.get_exchange_name(),
                                                    instmt.get_instmt_name())
            self.init_trades_table(instmt)
            id_ret = self.db_client.select(table=table_name,
                                           columns=['id'],
                                           orderby="id desc",
                                           limit=1)
            trade_id_ret = self.db_client.select(table=table_name,
                                                 columns=['id','trade_id'],
                                                 orderby="id desc",
                                                 limit=1)

            if len(id_ret) > 0 and len(trade_id_ret) > 0:
                assert isinstance(id_ret[0][0], int), "id_ret[0][0](%s) = %s" % (type(id_ret[0][0]), id_ret)
                return int(id_ret[0][0]), trade_id_ret[0][1][25:]
            else:
                return 0, '0'
        else:
            return 0, '0'

    def get_order_book_worker(self, instmt):
        """
        Get order book worker
        :param instmt: Instrument
        """
        instmt.set_order_book_id(self.get_order_book_init(instmt))

        while True:
            try:
                l2_depth = self.api_socket.get_order_book(instmt)
                if l2_depth is not None and l2_depth.is_diff(instmt.get_l2_depth()):
                    instmt.set_prev_l2_depth(instmt.l2_depth.copy())
                    instmt.set_l2_depth(l2_depth)
                    instmt.incr_order_book_id()
                    self.insert_order_book(instmt)
            except Exception as e:
                Logger.error(self.__class__.__name__,
                          "Error in order book: %s" % e)
            time.sleep(0.5)

    def get_trades_worker(self, instmt):
        """
        Get order book worker thread
        :param instmt: Instrument name
        """
        trade_id, last_exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(last_exch_trade_id)

        while True:
            try:
                ret = self.api_socket.get_trades(instmt)
                for trade in ret:
                    instmt.incr_trade_id()
                    self.insert_trade(instmt, trade)
                    
                # After the first time of getting the trade, indicate the instrument
                # is recovered
                if not instmt.get_recovered():
                    instmt.set_recovered(True)
                    
            except Exception as e:
                Logger.error(self.__class__.__name__,
                          "Error in trades: %s\nReturn: %s" % (e, ret))
            time.sleep(0.5)

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_prev_l2_depth(L2Depth(5))
        instmt.set_l2_depth(L2Depth(5))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                         instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                                 instmt.get_instmt_name()))

        t1 = threading.Thread(target=partial(self.get_order_book_worker, instmt))
        t1.start()
        t2 = threading.Thread(target=partial(self.get_trades_worker, instmt))
        t2.start()
        return [t1, t2]






from distutils.core import setup

setup(
    name='BitcoinExchangeFH',
    version='0.1.1rc3',
    author='Gavin Chan',
    author_email='gavincyi@gmail.com',
    packages=['befh'],
    url='http://pypi.python.org/pypi/BitcoinExchangeFH/',
    license='LICENSE.txt',
    description='Cryptocurrency exchange market data feed handler.',
    entry_points={
            'console_scripts': ['bitcoinexchangefh=befh.bitcoinexchangefh:main']
        },
    install_requires=[
            'pymysql',
            'websocket',
            'websocket-client',
            'socketIO_client==0.5.6',
            'numpy',
            'qpython',
            'pyzmq'
        ]
    )


from befh.api_socket import ApiSocket
try:
    import urllib.request as urlrequest
except ImportError:
    import urllib as urlrequest

import json

class RESTfulApiSocket(ApiSocket):
    """
    Generic REST API call
    """
    def __init__(self):
        """
        Constructor
        """
        ApiSocket.__init__(self)

    @classmethod
    def request(cls, url):
        """
        Web request
        :param: url: The url link
        :return JSON object
        """
        req = urlrequest.Request(url, headers={'User-Agent': 'Mozilla/5.0'})
        # res = urlrequest.urlopen(url)
        res = urlrequest.urlopen(req)
        try:
            res = json.loads(res.read().decode('utf8'))
            return res
        except:
            return {}
        
    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        return None

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        return None

    @classmethod
    def get_order_book(cls, instmt):
        """
        Get order book
        :param instmt: Instrument
        :return: Object L2Depth
        """
        return None

    @classmethod
    def get_trades(cls, instmt, trade_id):
        """
        Get trades
        :param instmt: Instrument
        :param trade_id: Trade id
        :return: List of trades
        """
        return None




from befh.sql_client import SqlClient
import pymysql

class MysqlClient(SqlClient):
    """
    Sqlite client
    """
    def __init__(self):
        """
        Constructor
        """
        SqlClient.__init__(self)

    def connect(self, **kwargs):
        """
        Connect
        :param path: sqlite file to connect
        """
        host = kwargs['host']
        port = kwargs['port']
        user = kwargs['user']
        pwd = kwargs['pwd']
        schema = kwargs['schema']
        self.conn = pymysql.connect(host=host,
                                    port=port,
                                    user=user,
                                    password=pwd,
                                    db=schema,
                                    charset='utf8mb4',
                                    cursorclass=pymysql.cursors.DictCursor)
        self.cursor = self.conn.cursor()
        return self.conn is not None and self.cursor is not None

    def execute(self, sql):
        """
        Execute the sql command
        :param sql: SQL command
        """
        self.cursor.execute(sql)

    def commit(self):
        """
        Commit
        """
        self.conn.commit()

    def fetchone(self):
        """
        Fetch one record
        :return Record
        """
        return self.cursor.fetchone()

    def fetchall(self):
        """
        Fetch all records
        :return Record
        """
        return self.cursor.fetchall()

    def select(self, table, columns=['*'], condition='', orderby='', limit=0, isFetchAll=True):
        """
        Select rows from the table
        :param table: Table name
        :param columns: Selected columns
        :param condition: Where condition
        :param orderby: Order by condition
        :param limit: Rows limit
        :param isFetchAll: Indicator of fetching all
        :return Result rows
        """
        select = SqlClient.select(self, table, columns, condition, orderby, limit, isFetchAll)
        if len(select) > 0:
            if columns[0] != '*':
                ret = []
                for ele in select:
                        row = []
                        for column in columns:
                            row.append(ele[column])

                        ret.append(row)
            else:
                ret = [list(e.values()) for e in select]

            return ret
        else:
            return select



from befh.restful_api_socket import RESTfulApiSocket
from befh.exchange import ExchangeGateway
from befh.market_data import L2Depth, Trade
from befh.util import Logger
from befh.instrument import Instrument
from befh.sql_client_template import SqlClientTemplate
import time
import threading
from functools import partial
from datetime import datetime


class ExchGwBtccRestfulApi(RESTfulApiSocket):
    """
    Exchange gateway BTCC RESTfulApi
    """
    def __init__(self):
        RESTfulApiSocket.__init__(self)
        
    @classmethod
    def get_timestamp_offset(cls):
        return 1
        
    @classmethod
    def get_order_book_timestamp_field_name(cls):
        return ''
        
    @classmethod
    def get_trades_timestamp_field_name(cls):
        return ''
        
    @classmethod
    def get_bids_field_name(cls):
        return ''
        
    @classmethod
    def get_asks_field_name(cls):
        return ''
        
    @classmethod
    def get_trade_side_field_name(cls):
        return ''
        
    @classmethod
    def get_trade_id_field_name(cls):
        return ''
        
    @classmethod
    def get_trade_price_field_name(cls):
        return ''        
        
    @classmethod
    def get_trade_volume_field_name(cls):
        return ''        
        
    @classmethod
    def get_order_book_link(cls, instmt):
        return ""

    @classmethod
    def get_trades_link(cls, instmt, trade_id=''):
        return ""
                
    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        l2_depth = L2Depth()
        keys = list(raw.keys())
        if cls.get_order_book_timestamp_field_name() in keys and \
           cls.get_bids_field_name() in keys and \
           cls.get_asks_field_name() in keys:
            
            # Date time
            date_time = float(raw[cls.get_order_book_timestamp_field_name()])
            date_time = date_time / cls.get_timestamp_offset()
            l2_depth.date_time = datetime.utcfromtimestamp(date_time).strftime("%Y%m%d %H:%M:%S.%f")
            
            # Bids
            bids = raw[cls.get_bids_field_name()]
            bids = sorted(bids, key=lambda x: x[0], reverse=True)
            for i in range(0, min(5, len(bids))):
                l2_depth.bids[i].price = float(bids[i][0]) if type(bids[i][0]) != float else bids[i][0]
                l2_depth.bids[i].volume = float(bids[i][1]) if type(bids[i][1]) != float else bids[i][1]   
                
            # Asks
            asks = raw[cls.get_asks_field_name()]
            asks = sorted(asks, key=lambda x: x[0])
            for i in range(0, min(5, len(asks))):
                l2_depth.asks[i].price = float(asks[i][0]) if type(asks[i][0]) != float else asks[i][0]
                l2_depth.asks[i].volume = float(asks[i][1]) if type(asks[i][1]) != float else asks[i][1]            
        else:
            raise Exception('Does not contain order book keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))
        
        return l2_depth

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        trade = Trade()
        keys = list(raw.keys())
        
        if cls.get_trades_timestamp_field_name() in keys and \
           cls.get_trade_id_field_name() in keys and \
           cls.get_trade_price_field_name() in keys and \
           cls.get_trade_volume_field_name() in keys:
        
            # Date time
            date_time = float(raw[cls.get_trades_timestamp_field_name()])
            date_time = date_time / cls.get_timestamp_offset()
            trade.date_time = datetime.utcfromtimestamp(date_time).strftime("%Y%m%d %H:%M:%S.%f")      
            
            # Trade side
            trade.trade_side = 1
                
            # Trade id
            trade.trade_id = str(raw[cls.get_trade_id_field_name()])
            
            # Trade price
            trade.trade_price = float(str(raw[cls.get_trade_price_field_name()]))
            
            # Trade volume
            trade.trade_volume = float(str(raw[cls.get_trade_volume_field_name()]))
        else:
            raise Exception('Does not contain trade keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))        

        return trade

    @classmethod
    def get_order_book(cls, instmt):
        """
        Get order book
        :param instmt: Instrument
        :return: Object L2Depth
        """
        res = cls.request(cls.get_order_book_link(instmt))
        if len(res) > 0:
            return cls.parse_l2_depth(instmt=instmt,
                                       raw=res)
        else:
            return None

    @classmethod
    def get_trades(cls, instmt):
        """
        Get trades
        :param instmt: Instrument
        :param trade_id: Trade id
        :return: List of trades
        """
        link = cls.get_trades_link(instmt)
        res = cls.request(link)
        trades = []
        if len(res) > 0:
            for t in res:
                trade = cls.parse_trade(instmt=instmt,
                                         raw=t)
                trades.append(trade)

        return trades


class ExchGwBtcc(ExchangeGateway):
    """
    Exchange gateway BTCC
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwBtccRestfulApi(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'BTCC'

    def get_order_book_worker(self, instmt):
        """
        Get order book worker
        :param instmt: Instrument
        """
        instmt.set_order_book_id(self.get_order_book_init(instmt))

        while True:
            try:
                l2_depth = self.api_socket.get_order_book(instmt)
                if l2_depth is not None and l2_depth.is_diff(instmt.get_l2_depth()):
                    instmt.set_prev_l2_depth(instmt.get_l2_depth())
                    instmt.set_l2_depth(l2_depth)
                    instmt.incr_order_book_id()
                    self.insert_order_book(instmt)
            except Exception as e:
                Logger.error(self.__class__.__name__, "Error in order book: %s" % e)
            time.sleep(1)

    def get_trades_worker(self, instmt):
        """
        Get order book worker thread
        :param instmt: Instrument name
        """
        trade_id, exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(exch_trade_id)

        while True:
            try:
                ret = self.api_socket.get_trades(instmt)
                if ret is None or len(ret) == 0:
                    time.sleep(1)
                    continue
            except Exception as e:
                Logger.error(self.__class__.__name__, "Error in trades: %s" % e)                
                
            for trade in ret:
                assert isinstance(trade.trade_id, str), "trade.trade_id(%s) = %s" % (type(trade.trade_id), trade.trade_id)
                assert isinstance(instmt.get_exch_trade_id(), str), \
                       "instmt.get_exch_trade_id()(%s) = %s" % (type(instmt.get_exch_trade_id()), instmt.get_exch_trade_id())
                if int(trade.trade_id) > int(instmt.get_exch_trade_id()):
                    instmt.set_exch_trade_id(trade.trade_id)
                    instmt.incr_trade_id()
                    self.insert_trade(instmt, trade)
            
            # After the first time of getting the trade, indicate the instrument
            # is recovered
            if not instmt.get_recovered():
                instmt.set_recovered(True)

            time.sleep(1)

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_l2_depth(L2Depth(5))
        instmt.set_prev_l2_depth(L2Depth(5))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                        instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                                instmt.get_instmt_name()))
        instmt.set_recovered(False)
        t1 = threading.Thread(target=partial(self.get_order_book_worker, instmt))
        t1.start()
        t2 = threading.Thread(target=partial(self.get_trades_worker, instmt))
        t2.start()
        return [t1, t2]


class ExchGwBtccSpotRestfulApi(ExchGwBtccRestfulApi):
    """
    Exchange gateway Spot Instrument RESTful API
    """
    def __init__(self):
        ExchGwBtccRestfulApi.__init__(self)
        
    @classmethod
    def get_timestamp_offset(cls):
        return 1
        
    @classmethod
    def get_order_book_timestamp_field_name(cls):
        return 'date'
        
    @classmethod
    def get_trades_timestamp_field_name(cls):
        return 'date'
    
    @classmethod
    def get_bids_field_name(cls):
        return 'bids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'asks'
        
    @classmethod
    def get_trade_side_field_name(cls):
        return 'type'
        
    @classmethod
    def get_trade_id_field_name(cls):
        return 'tid'
        
    @classmethod
    def get_trade_price_field_name(cls):
        return 'price'        
        
    @classmethod
    def get_trade_volume_field_name(cls):
        return 'amount'        
        
    @classmethod
    def get_order_book_link(cls, instmt):
        return "https://data.btcchina.com/data/orderbook?limit=5&market=%s" % instmt.get_instmt_code()

    @classmethod
    def get_trades_link(cls, instmt):
        if int(instmt.get_exch_trade_id()) > 0:
            return "https://data.btcchina.com/data/historydata?market=%s&since=%s" % \
                (instmt.get_instmt_code(), instmt.get_exch_trade_id())
        else:
            return "https://data.btcchina.com/data/historydata?limit=100&market=%s" % \
                (instmt.get_instmt_code())        

class ExchGwBtccFutureRestfulApi(ExchGwBtccRestfulApi):
    """
    Exchange gateway Spot Instrument RESTful API
    """
    def __init__(self):
        ExchGwBtccRestfulApi.__init__(self)
        
    @classmethod
    def get_timestamp_offset(cls):
        return 1000
        
    @classmethod
    def get_order_book_timestamp_field_name(cls):
        return 'date'
        
    @classmethod
    def get_trades_timestamp_field_name(cls):
        return 'Timestamp'
    
    @classmethod
    def get_bids_field_name(cls):
        return 'bids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'asks'
        
    @classmethod
    def get_trade_side_field_name(cls):
        return 'Side'
        
    @classmethod
    def get_trade_id_field_name(cls):
        return 'Id'
        
    @classmethod
    def get_trade_price_field_name(cls):
        return 'Price'        
        
    @classmethod
    def get_trade_volume_field_name(cls):
        return 'Quantity'        
        
    @classmethod
    def get_order_book_link(cls, instmt):
        return "https://pro-data.btcc.com/data/pro/orderbook?limit=5&symbol=%s" % instmt.get_instmt_code()

    @classmethod
    def get_trades_link(cls, instmt):
        if int(instmt.get_exch_trade_id()) > 0:
            return "https://pro-data.btcc.com/data/pro/historydata?symbol=%s&since=%s" % \
                (instmt.get_instmt_code(), instmt.get_exch_trade_id())
        else:
            return "https://pro-data.btcc.com/data/pro/historydata?limit=100&symbol=%s" % \
                (instmt.get_instmt_code())        


class ExchGwBtccSpot(ExchGwBtcc):
    """
    Exchange gateway BTCC-Spot
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwBtccSpotRestfulApi(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'BTCC_Spot'
        
        
class ExchGwBtccFuture(ExchGwBtcc):
    """
    Exchange gateway BTCC-Future
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwBtccFutureRestfulApi(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'BTCC_Future'        
        
        
if __name__ == '__main__':
    Logger.init_log()
    exchange_name = 'BTCC_Spot'
    instmt_name = 'BTCCNY'
    instmt_code = 'btccny'
    instmt = Instrument(exchange_name, instmt_name, instmt_code)    
    db_client = SqlClientTemplate()
    exch = ExchGwBtccSpot(db_client)
    instmt.set_l2_depth(L2Depth(5))
    instmt.set_prev_l2_depth(L2Depth(5))
    instmt.set_order_book_table_name(exch.get_order_book_table_name(instmt.get_exchange_name(),
                                                                    instmt.get_instmt_name()))
    instmt.set_trades_table_name(exch.get_trades_table_name(instmt.get_exchange_name(),
                                                            instmt.get_instmt_name()))
    instmt.set_recovered(False)    
    exch.get_order_book_worker(instmt)


from befh.ws_api_socket import WebSocketApiClient
from befh.market_data import L2Depth, Trade
from befh.exchange import ExchangeGateway
from befh.instrument import Instrument
from befh.util import Logger
import time
import threading
import json
from functools import partial
from datetime import datetime


class ExchGwOkCoinWs(WebSocketApiClient):
    """
    Exchange socket
    """
    def __init__(self):
        """
        Constructor
        """
        WebSocketApiClient.__init__(self, 'ExchGwOkCoin')
        
    @classmethod
    def get_order_book_timestamp_field_name(cls):
        return 'timestamp'
    
    @classmethod
    def get_bids_field_name(cls):
        return 'bids'
        
    @classmethod
    def get_asks_field_name(cls):
        return 'asks'
        
    @classmethod
    def get_link(cls):
        return 'wss://real.okcoin.com:10440/websocket/okcoinapi'

    @classmethod
    def get_order_book_subscription_string(cls, instmt):
        return json.dumps({"event":"addChannel", "channel": instmt.get_order_book_channel_id()})

    @classmethod
    def get_trades_subscription_string(cls, instmt):
        return json.dumps({"event":"addChannel", "channel": instmt.get_trades_channel_id()})

    @classmethod
    def parse_l2_depth(cls, instmt, raw):
        """
        Parse raw data to L2 depth
        :param instmt: Instrument
        :param raw: Raw data in JSON
        """
        l2_depth = instmt.get_l2_depth()
        keys = list(raw.keys())
        if cls.get_order_book_timestamp_field_name() in keys and \
           cls.get_bids_field_name() in keys and \
           cls.get_asks_field_name() in keys:
            
            # Date time
            timestamp = float(raw[cls.get_order_book_timestamp_field_name()])/1000.0
            l2_depth.date_time = datetime.utcfromtimestamp(timestamp).strftime("%Y%m%d %H:%M:%S.%f")
            
            # Bids
            bids = raw[cls.get_bids_field_name()]
            bids = sorted(bids, key=lambda x: x[0], reverse=True)
            for i in range(0, len(bids)):
                l2_depth.bids[i].price = float(bids[i][0]) if type(bids[i][0]) != float else bids[i][0]
                l2_depth.bids[i].volume = float(bids[i][1]) if type(bids[i][1]) != float else bids[i][1]   
                
            # Asks
            asks = raw[cls.get_asks_field_name()]
            asks = sorted(asks, key=lambda x: x[0])
            for i in range(0, len(asks)):
                l2_depth.asks[i].price = float(asks[i][0]) if type(asks[i][0]) != float else asks[i][0]
                l2_depth.asks[i].volume = float(asks[i][1]) if type(asks[i][1]) != float else asks[i][1]            
        else:
            raise Exception('Does not contain order book keys in instmt %s-%s.\nOriginal:\n%s' % \
                (instmt.get_exchange_name(), instmt.get_instmt_name(), \
                 raw))        

        return l2_depth

    @classmethod
    def parse_trade(cls, instmt, raw):
        """
        :param instmt: Instrument
        :param raw: Raw data in JSON
        :return:
        """
        trade = Trade()
        trade_id = raw[0]
        trade_price = float(raw[1])
        trade_volume = float(raw[2])
        timestamp = raw[3]
        trade_side = raw[4]
        
        trade.trade_id = trade_id + timestamp
        trade.trade_price = trade_price
        trade.trade_volume = trade_volume
        trade.trade_side = Trade.parse_side(trade_side)

        return trade


class ExchGwOkCoin(ExchangeGateway):
    """
    Exchange gateway
    """
    def __init__(self, db_client):
        """
        Constructor
        :param db_client: Database client
        """
        ExchangeGateway.__init__(self, ExchGwOkCoinWs(), db_client)

    @classmethod
    def get_exchange_name(cls):
        """
        Get exchange name
        :return: Exchange name string
        """
        return 'OkCoin'

    def on_open_handler(self, instmt, ws):
        """
        Socket on open handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is subscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        if not instmt.get_subscribed():
            instmt_code_split = instmt.get_instmt_code().split('_')
            if len(instmt_code_split) == 3:
                # Future instruments
                instmt.set_order_book_channel_id("ok_sub_%s_%s_depth_%s_20" % \
                                                 (instmt_code_split[0],
                                                  instmt_code_split[1],
                                                  instmt_code_split[2]))
                instmt.set_trades_channel_id("ok_sub_%s_%s_trade_%s" % \
                                               (instmt_code_split[0],
                                                instmt_code_split[1],
                                                instmt_code_split[2]))
            else:
                # Spot instruments
                instmt.set_order_book_channel_id("ok_sub_%s_depth_20" % instmt.get_instmt_code())
                instmt.set_trades_channel_id("ok_sub_%s_trades" % instmt.get_instmt_code())

            ws.send(self.api_socket.get_order_book_subscription_string(instmt))
            ws.send(self.api_socket.get_trades_subscription_string(instmt))
            instmt.set_subscribed(True)

    def on_close_handler(self, instmt, ws):
        """
        Socket on close handler
        :param instmt: Instrument
        :param ws: Web socket
        """
        Logger.info(self.__class__.__name__, "Instrument %s is unsubscribed in channel %s" % \
                  (instmt.get_instmt_code(), instmt.get_exchange_name()))
        instmt.set_subscribed(False)

    def on_message_handler(self, instmt, messages):
        """
        Incoming message handler
        :param instmt: Instrument
        :param message: Message
        """
        for message in messages:
            keys = message.keys()
            if 'channel' in keys:
                if 'data' in keys:
                    if message['channel'] == instmt.get_order_book_channel_id():
                        data = message['data']
                        instmt.set_prev_l2_depth(instmt.get_l2_depth().copy())
                        self.api_socket.parse_l2_depth(instmt, data)

                        # Insert only if the first 5 levels are different
                        if instmt.get_l2_depth().is_diff(instmt.get_prev_l2_depth()):
                            instmt.incr_order_book_id()
                            self.insert_order_book(instmt)

                    elif message['channel'] == instmt.get_trades_channel_id():
                        for trade_raw in message['data']:
                            trade = self.api_socket.parse_trade(instmt, trade_raw)
                            if trade.trade_id != instmt.get_exch_trade_id():
                                instmt.incr_trade_id()
                                instmt.set_exch_trade_id(trade.trade_id)
                                self.insert_trade(instmt, trade)

                elif 'success' in keys:
                    Logger.info(self.__class__.__name__, "Subscription to channel %s is %s" \
                        % (message['channel'], message['success']))
            else:
                Logger.info(self.__class__.__name__, ' - ' + json.dumps(message))

    def start(self, instmt):
        """
        Start the exchange gateway
        :param instmt: Instrument
        :return List of threads
        """
        instmt.set_prev_l2_depth(L2Depth(20))
        instmt.set_l2_depth(L2Depth(20))
        instmt.set_order_book_table_name(self.get_order_book_table_name(instmt.get_exchange_name(),
                                                                       instmt.get_instmt_name()))
        instmt.set_trades_table_name(self.get_trades_table_name(instmt.get_exchange_name(),
                                                               instmt.get_instmt_name()))
        instmt.set_order_book_id(self.get_order_book_init(instmt))
        trade_id, last_exch_trade_id = self.get_trades_init(instmt)
        instmt.set_trade_id(trade_id)
        instmt.set_exch_trade_id(last_exch_trade_id)
        return [self.api_socket.connect(self.api_socket.get_link(),
                                        on_message_handler=partial(self.on_message_handler, instmt),
                                        on_open_handler=partial(self.on_open_handler, instmt),
                                        on_close_handler=partial(self.on_close_handler, instmt))]




#!/bin/python

import argparse
import sys

from befh.exchange import ExchangeGateway
from befh.exch_bitmex import ExchGwBitmex
from befh.exch_btcc import ExchGwBtccSpot, ExchGwBtccFuture
from befh.exch_bitfinex import ExchGwBitfinex
from befh.exch_okcoin import ExchGwOkCoin
from befh.exch_kraken import ExchGwKraken
from befh.exch_huobi import ExchGwHuobi
from befh.exch_gdax import ExchGwGdax
from befh.exch_bitstamp import ExchGwBitstamp
from befh.exch_gatecoin import ExchGwGatecoin
from befh.exch_quoine import ExchGwQuoine
from befh.kdbplus_client import KdbPlusClient
from befh.mysql_client import MysqlClient
from befh.sqlite_client import SqliteClient
from befh.file_client import FileClient
from befh.zmq_client import ZmqClient
from befh.subscription_manager import SubscriptionManager
from befh.util import Logger


def main():
    parser = argparse.ArgumentParser(description='Bitcoin exchange market data feed handler.')
    parser.add_argument('-mode', action='store', help='Mode. Supports ALL, ORDER_BOOK_AND_TRADES_ONLY, SNAPSHOT_ONLY, ORDER_BOOK_ONLY and TRADES_ONLY', default='ALL')
    parser.add_argument('-instmts', action='store', help='Instrument subscription file.', default='subscriptions.ini')
    parser.add_argument('-exchtime', action='store_true', help='Use exchange timestamp.')
    parser.add_argument('-kdb', action='store_true', help='Use Kdb+ as database.')
    parser.add_argument('-csv', action='store_true', help='Use csv file as database.')
    parser.add_argument('-sqlite', action='store_true', help='Use SQLite database.')
    parser.add_argument('-mysql', action='store_true', help='Use MySQL.')
    parser.add_argument('-zmq', action='store_true', help='Use zmq publisher.')
    parser.add_argument('-dbpath', action='store', dest='dbpath',
                        help='Database file path. Supported for SQLite only.',
                        default='bitcoinexchange.raw')
    parser.add_argument('-dbdir', action='store', dest='dbdir',
                        help='Database file directory. Supported for CSV only.',
                        default='')
    parser.add_argument('-dbaddr', action='store', dest='dbaddr', default='localhost',
                        help='Database address. Defaulted as localhost. Supported for database with connection')
    parser.add_argument('-dbport', action='store', dest='dbport', default='3306',
                        help='Database port, Defaulted as 3306. Supported for database with connection')
    parser.add_argument('-dbuser', action='store', dest='dbuser',
                        help='Database user. Supported for database with connection')
    parser.add_argument('-dbpwd', action='store', dest='dbpwd',
                        help='Database password. Supported for database with connection')
    parser.add_argument('-dbschema', action='store', dest='dbschema',
                        help='Database schema. Supported for database with connection')
    parser.add_argument('-output', action='store', dest='output',
                        help='Verbose output file path')
    args = parser.parse_args()

    Logger.init_log(args.output)

    if args.sqlite:
        db_client = SqliteClient()
        db_client.connect(path=args.dbpath)
    elif args.mysql:
        db_client = MysqlClient()
        db_client.connect(host=args.dbaddr,
                          port=int(args.dbport),
                          user=args.dbuser,
                          pwd=args.dbpwd,
                          schema=args.dbschema)
    elif args.csv:
        if args.dbdir != '':
            db_client = FileClient(dir=args.dbdir)
        else:
            db_client = FileClient()
    elif args.kdb:
        db_client = KdbPlusClient()
        db_client.connect(host=args.dbaddr, port=int(args.dbport))
    elif args.zmq:
        db_client = ZmqClient()
        db_client.connect(addr=args.dbaddr)
    else:
        print('Error: Please define which database is used.')
        parser.print_help()
        sys.exit(1)

    # Subscription instruments
    if args.instmts is None or len(args.instmts) == 0:
        print('Error: Please define the instrument subscription list. You can refer to subscriptions.ini.')
        parser.print_help()
        sys.exit(1)
        
    # Data mode
    if args.mode is not None:
        ExchangeGateway.data_mode = ExchangeGateway.DataMode.fromstring(args.mode)
        
    # Use exchange timestamp rather than local timestamp
    if args.exchtime:
        ExchangeGateway.is_local_timestamp = False

    subscription_instmts = SubscriptionManager(args.instmts).get_subscriptions()
    ExchangeGateway.init_snapshot_table(ExchangeGateway.data_mode, db_client)

    Logger.info('[main]', 'Current mode = %s' % args.mode)
    Logger.info('[main]', 'Subscription file = %s' % args.instmts)
    

    exch_gws = []
    exch_gws.append(ExchGwBtccSpot(db_client))
    exch_gws.append(ExchGwBtccFuture(db_client))
    exch_gws.append(ExchGwBitmex(db_client))
    exch_gws.append(ExchGwBitfinex(db_client))
    exch_gws.append(ExchGwOkCoin(db_client))
    exch_gws.append(ExchGwKraken(db_client))
    exch_gws.append(ExchGwHuobi(db_client))
    exch_gws.append(ExchGwGdax(db_client))
    exch_gws.append(ExchGwBitstamp(db_client))
    exch_gws.append(ExchGwGatecoin(db_client))
    exch_gws.append(ExchGwQuoine(db_client))
    threads = []
    for exch in exch_gws:
        for instmt in subscription_instmts:
            if instmt.get_exchange_name() == exch.get_exchange_name():
                Logger.info("[main]", "Starting instrument %s-%s..." % \
                    (instmt.get_exchange_name(), instmt.get_instmt_name()))
                threads += exch.start(instmt)

if __name__ == '__main__':
    main()